from __future__ import annotations

from tcsoa.gen.CalendarManagement._2007_06.CalendarManagement import CalendarUpdate, CreateCalendarResponse, CalendarContainer
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class CalendarManagementService(TcService):

    @classmethod
    def updateCalendars(cls, tccalendarUpdates: List[CalendarUpdate]) -> ServiceData:
        """
        Updates the specified calendars and their events.
        This includes creating, modifying and deleting events associated with specified calendars
        """
        return cls.execute_soa_method(
            method_name='updateCalendars',
            library='CalendarManagement',
            service_date='2007_06',
            service_name='CalendarManagement',
            params={'tccalendarUpdates': tccalendarUpdates},
            response_cls=ServiceData,
        )

    @classmethod
    def createCalendars(cls, newTCCalendars: List[CalendarContainer]) -> CreateCalendarResponse:
        """
        Creates new calendars based on the provided input parameters
        """
        return cls.execute_soa_method(
            method_name='createCalendars',
            library='CalendarManagement',
            service_date='2007_06',
            service_name='CalendarManagement',
            params={'newTCCalendars': newTCCalendars},
            response_cls=CreateCalendarResponse,
        )



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from tcsoa.gen.BusinessObjects import WorkspaceObject
from dataclasses import dataclass
from typing import Dict


@dataclass
class MarkupResponse(TcBaseObj):
    """
    This MarkupResponse structure is for the response of all processMarkups operations.
    
    :var baseObject: The resulting base object after process markups.
    :var version: The version of the markups set by the server, opaque to the client.
    :var properties: A map (string/string) of properties.    Values can be:
    - Key: "message".  Value: "author","reviewer" or "editor"
    
    
    :var markups: The user supplied JSON markup data that was updated or loaded.
    :var serviceData: The Service Data that may contain partial errors.
    """
    baseObject: WorkspaceObject = None
    version: str = ''
    properties: MarkupProperties = None
    markups: str = ''
    serviceData: ServiceData = None


"""
Contains the key/value pairs for client/server communication.
"""
MarkupProperties = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine, BOMWindow
from tcsoa.gen.StructureManagement._2011_06.Structure import ValidateStructureItemIdsInfo3, UnloadType, ExpandOrUpdateDuplicateItemsResponse2, DuplicateInputInfo3, ValidateStructureItemIdsResponse3, ClosureRuleVariableInfo
from tcsoa.gen.StructureManagement._2008_06.Structure import ExpandOrUpdateDuplicateItemsInfo, DuplicateResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class StructureService(TcService):

    @classmethod
    def setClosureRuleVariablesAndValues(cls, window: BOMWindow, closureRuleName: str, closureRuleVariableInfos: List[ClosureRuleVariableInfo]) -> ServiceData:
        """
        Sets ClosureRule on the BOMWindow. If ClosureRule has variables then respective values supplied are used during
        ClosureRule evaluation. Only ClosureRule with scope PIE_TEAMCENTER is considered by this operation.
        
        In case if ClosureRule should be reset on BOMWindow then input ClosureRule name should be empty string.
        
        Use cases:
        Perform controlled BOM expansion using ClosureRule.
        """
        return cls.execute_soa_method(
            method_name='setClosureRuleVariablesAndValues',
            library='StructureManagement',
            service_date='2011_06',
            service_name='Structure',
            params={'window': window, 'closureRuleName': closureRuleName, 'closureRuleVariableInfos': closureRuleVariableInfos},
            response_cls=ServiceData,
        )

    @classmethod
    def unloadBelow(cls, bomlines: List[BOMLine], unloadType: UnloadType) -> ServiceData:
        """
        This operation unloads and destroy the BOM structure ( i.e. BOMLine objects) below supplied BOMLine object or
        objects. It would unload associated persistence objects like Item, ItemRevision, PSBomViewrevision and
        PSOccurrence etc. and free up memory.
        
        Use cases:
        The operation is intended to improve the scalability of BOM structure expansion. From a large BOM structure,
        user can unload the BOM structure that he has finished working on. This will free up the memory which would be
        used for expanding another sub-structure that user starts working on.
        """
        return cls.execute_soa_method(
            method_name='unloadBelow',
            library='StructureManagement',
            service_date='2011_06',
            service_name='Structure',
            params={'bomlines': bomlines, 'unloadType': unloadType},
            response_cls=ServiceData,
        )

    @classmethod
    def validateStructureItemIds3(cls, inputs: List[ValidateStructureItemIdsInfo3]) -> ValidateStructureItemIdsResponse3:
        """
        This operation is called as part of the duplicate/clone operation.  It will validate the un-validated ItemIds
        and user selected projects that will be used to clone(duplicate) an assembly structure.   All portions of the
        structure that are displayed in the duplicate dialog have been validated by the client.
        
        Note: The differences between the three operations 'validateStructureItemIds', 'validateStructureItemIds2' and
        'validateStructureItemIds3' are the following:
        - In 'validateStructureItemIds' the input is the top BOMLine of the original configured structure in Structure
        Manager.  In 'validateStructureItemIds2' the input is the selected BOMLine of the configured structure in
        Structure Manager or Systems Engineering.  i.e. the user can select to clone a sub-assembly of the original
        structure or the entire assembly.  The input for 'validateStructureItemIds2' includes project(s).  The cloned
        structure is assigned to those project(s).  'validateStructureItemIds' did not have projects as input.
        
        - 'validateStructureItemIds3' was created as a result of the Multi Field Key(MFK) project.  The difference
        between 'validateStructureItemIds2' and 'validateStructureItemIds3' is that in 'validateStructureItemIds2' the
        input and output have a 'DuplicateIdMap2' whereas to align with the MFK project, 'validateStructureItemIds3'
        has a 'DuplicateIdStructure'.
        
        Use cases:
        The user sends in a structure for validation of the new ItemIds.  The input to this operation is the top
        BOMLine or the selected BOMLine, a structure of old item components to new ItemId and new ItemName for those
        Item objects that are already validated, the default naming scheme and a list of user defined projects to which
        the duplicated Item objects will be added.  Based on the structure traversal, the input structure, and the
        naming scheme this operation validates whether the proposed names for the un-validated Item objects are valid
        and whether the user can add the new structure to the list of user defined projects.
        """
        return cls.execute_soa_method(
            method_name='validateStructureItemIds3',
            library='StructureManagement',
            service_date='2011_06',
            service_name='Structure',
            params={'inputs': inputs},
            response_cls=ValidateStructureItemIdsResponse3,
        )

    @classmethod
    def duplicate3(cls, inputs: List[DuplicateInputInfo3]) -> DuplicateResponse:
        """
        This operation will create a duplicate (clone) of the input structure from its top level down or a selected sub
        assembly. 
         
        Depending on the user action, all or some of the original structure is duplicated and the rest is referenced. 
        The user has the option to de-select a sub assembly or leaf Item on the duplicate dialog.  Those Item objects
        that are not selected in the duplicate dialog will not be cloned but referenced from the original structure.  
        
        The user can define a specific naming pattern for the ItemIds of the duplicated (cloned) structure.  The user
        can define specific ItemIds for individually selected ItemRevision objects or a default naming pattern for the
        duplicated structure.   The default pattern can be defined by adding prefixes, suffixes or replacing part of
        the original name with a different pattern.  The user can also choose to allow the system to assign default ids.
        
        If project(s) have been passed in as input, the cloned structure is assigned to that project(s).  By default
        the projects to which the top BOMLine in the duplicate dialog belongs and to which the user has access, is used
        to populate the project list.  The user has the option to modify that list based on which projects are
        available to that user.
        
        All of the structure dependent data of the input structure like datasets and attachments are copied to the
        duplicated structure based on the Business Modeler IDE deep copy rules for 'SaveAs'.  The duplicate operation
        internally uses 'SaveAs' at every level of the structure; therefore it uses the 'SaveAs' deep copy rules.
        
        If the structure being cloned is a Requirements Manager structure, tracelink GRMs are handled based on the deep
        copy rules set for ReqRev for 'SaveAs'.  
        Target                             ReqRev
        Relation                           FND_TraceLink                
        Attached Type                ReqRev
        Operation                        SaveAs
        Action                             CopyAsReference
        Condition
        Direction                         IsTargetPrimary= false
        
        CAD specific attachments and relations are copied based on the transfer mode defined for the Business Modeler
        IDE global constant StructureCloneTransferModes.  The transfer mode contains dependent closure rules for
        expansion and cloning.  The value for the closure rules is added by the installed CAD system.
        For e.g. The closure rule defined for IPEM ProE integration (ipemSCloneClosureRule) looks like this:
        TYPE.ProPrt:CLASS.ItemRevision:RELATIONP2S.IPEM_master_dependency:PROCESS:PartFamilyMaster:R,
        TYPE.ProAsm:CLASS.ItemRevision:RELATIONP2S.IPEM_master_dependency:PROCESS:PartFamilyMaster:R,
        TYPE.ProPrt:CLASS.ItemRevision:RELATIONS2P.IPEM_master_dependency:PROCESS+TRAVERSE:PartFamilyMember, 
        
        Note: The difference between the three operations 'duplicate', 'duplicate2' and 'duplicate3' are the following:
        'duplicate' and 'duplicate2'
         - In 'duplicate' the input top BOMLine is the top BOMLine of the original configured structure in Structure
        Manager.  In 'duplicate2' the input top BOMLine is the selected BOMLine from the configured structure in
        Structure Manager or Systems Engineering.  i.e. the user can select to clone a sub-assembly of the original
        structure.
        - The input for 'duplicate2' includes project(s).  The cloned structure is assigned to those project(s). 
        'duplicate' does not have projects as input.
        
        'duplicate2' and 'duplicate3'
        - 'duplicate3' was created as a result of the Multi Field Key (MFK ) project.   The difference between
        'duplicate2' and 'duplicate3' is that in 'duplicate2' is that the input and output had a 'DuplicateIdMap2'
        whereas to align with the MFK project, 'duplicate3' has a 'DuplicateIdStructure'.
        
        
        Use cases:
        Use case1: Simple Clone
        A user has an assembly which does not have cad dependencies nor does it belong to a specific project(s).  The
        user wants to duplicate the entire structure with a specific naming pattern for the ItemIds.  The naming
        pattern is a prefix "test_".
        The user invokes the duplicate operation by passing in the top BOMLine of the structure to be cloned, and the
        naming pattern for the new structure.  The result is a new structure with the following naming pattern for the
        ItemIds -> test_OriginalItemId.
        
        Use case2: CAD Clone
        A user has an assembly structure which has cad dependencies.  The user wants to start a new product with a
        similar structure and cad dependencies.  The expansion and cloning rules have been defined in the Business
        Modeler IDE global constant StructureCloneTransferModes. 
        The user invokes the duplicate operation by passing in the top BOMLine of the structure to be cloned.  
        The user selects the cad dependency option Part Family Master.  The expansion and cloning will be done based on
        the closure rules defined for Part Family Master in the CAD closure rules.
        The "'Rename Cad Files'" will be passed to the CAD integration in a callback.  If the "'Rename Cad Files'", is
        set to true by the user, the cad files need to be renamed by the cad integration.
        The result will be a duplicated structure with the expected cad dependencies and it will open in the CAD tool
        without any errors.
        
        Use case3: Requirements Manager (Systems Engineering) clone:
        The user has a requirements manager structure with internal and/or external tracelink GRMs that need to be
        copied to the cloned structure.  The user defines a set of projects to which the newly cloned structure should
        belong.  The user does not want to clone the entire structure only a sub-assembly.
        The precondition to this operation, is that the deep copy rules for 'SaveAs' have been setup correctly
        The user invokes the duplicate operation by passing in the selected BOMLine of the sub structure to be cloned. 
        The projects to which the cloned structure should belong are passed in as input.  The naming rule for the
        ItemId is passed in.  
        The result is a requirement manager structure with the tracelink relations pointing to the correct objects in
        the new structure.  And the newly cloned structure belongs to the defined projects for which the user has
        permissions. 
        
        """
        return cls.execute_soa_method(
            method_name='duplicate3',
            library='StructureManagement',
            service_date='2011_06',
            service_name='Structure',
            params={'inputs': inputs},
            response_cls=DuplicateResponse,
        )

    @classmethod
    def expandOrUpdateDuplicateItems3(cls, opInput: List[ExpandOrUpdateDuplicateItemsInfo], selectionOption: int) -> ExpandOrUpdateDuplicateItemsResponse2:
        """
        This operation  is designed to expand structures one level at a time and get structure dependent data. When it
        is flagged for smart selection, it will try to solve the uncertain smart selection by expansion, in which case
        only qualified ItemRevision objects will be returned. This operation is used by the duplicate (clone) structure
        feature.  The dependencies have been defined to allow duplication of CAD dependent structure.  The
        'expandOrUpdateDuplicateItems3'  operation uses Business Modeler IDE global constant
        StructureCloneTransferModes to determine which  of the cad specific attachments and relationships can be
        expanded. This constant defines the transfer modes containing dependent closure rules for expansion and
        cloning.  The value for the closure rules is added by the installed CAD system.  For this version of the
        operation a mandatory clause has been added in the closure rules that removes the limit on one level of
        relationship traversal.   This was done to enhance the Duplicate functionality to ensure that the duplicated
        structure is CAD consistent.
        
        Note: The difference between 'expandOrUpdateDuplicateItems', 'expandOrUpdateDuplicateItems2' and
        'expandOrUpdateDuplicateItems3' is as follows:
        
        Difference in 'expandOrUpdateDuplicateItems' and 'expandOrUpdateDuplicateItems2'
        -    We allow the user to select a sub assembly for duplication.  There is a performance problem that was
        uncovered.  Even though a sub assembly is sent for duplication, traversal on the server side was happening from
        the top BOMLine of the structure.  To improve the performance we get a BOB object.
        
        -    The smart selection logic was added to decide whether to clone or reference an Item in a structure based
        on the project that the top line of the original structure belongs to.  This smart selection logic is bottom
        up, so if a child is selected based on project assignment, the parent will be selected, no matter whether the
        parent belongs to the top item project assignment or not.
        
        Difference in 'expandOrUpdateDuplicateItems2' and 'expandOrUpdateDuplicateItems3'
        The mandatory flag is passed back to the client.  When a cad option has been flagged with a mandatory flag, a
        "R" predicate in the closure rules, that option will come up in the Duplicate Dialog pre-checked and its
        selected status will be un-changeable.  This will prevent the user from un-checking those ItemRevision objects
        that were added in to make the structure cad consistent.  That is if a family table member in the assembly
        structure has been selected for duplication, then automatically include all its masters from the referenced
        ItemRevision all the way to the top master.  Including the masters automatically will only happen if the CAD
        closure rules are defined with a predicate "R" that says this CAD relation is mandatory for duplication. 
        
        
        Use cases:
        Use case 1: selectionOption is 0 and the original structure has cad data:
        The user sends in a structure for expansion, it will be expanded one level at a time and all dependent data
        will be returned based on the input and the value of the defined closure rule.  The input consists of the
        BOMLine for expansion and ItemRevision objects on which to perform the expansion,  the dependency types, and
        the selectionOption.  The ItemRevision objects could be null, in which case the ItemRevision object(s) gotten
        from the expansion of the BOMLine are used.  The dependency types are checked against the definition in the
        closure rules to determine  what dependent data is expanded. 
        
        Use case 2: selectionOption is 1 and the original structure has no cad data
        The user sends in a structure for expansion, it will be expanded one level at a time and all dependent data
        will be returned based on the input.  In this case no closure rule may be defined, since the structure has no
        cad data.   The input consists of the BOMLine for expansion and ItemRevision objects on which to perform the
        expansion,  the dependency types, and the selectionOption.  The ItemRevision objects could be null, in which
        case the ItemRevision object(s) gotten from the expansion of the BOMLine are used.  Since the selectionOption
        is 1, the input lines will be checked based on the top Use case 1: selectionOption is 0 and the original
        structure has cad data:
        The user sends in a structure for expansion, it will be expanded one level at a time and all dependent data
        will be returned based on the input and the value of the defined closure rule.  The input consists of the
        BOMLine for expansion and ItemRevision objects on which to perform the expansion,  the dependency types, and
        the selectionOption.  The ItemRevision objects could be null, in which case the ItemRevision object(s) gotten
        from the expansion of the BOMLine are used.  The dependency types are checked against the definition in the
        closure rules to determine  what dependent data is expanded. 
        """
        return cls.execute_soa_method(
            method_name='expandOrUpdateDuplicateItems3',
            library='StructureManagement',
            service_date='2011_06',
            service_name='Structure',
            params={'opInput': opInput, 'selectionOption': selectionOption},
            response_cls=ExpandOrUpdateDuplicateItemsResponse2,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Cfg0ConfiguratorPerspective
from tcsoa.gen.Configurator._2014_12.ConfiguratorManagement import GetAdmissibilityResponse, GetAdmissibilityInputStruct
from typing import List
from tcsoa.base import TcService


class ConfiguratorManagementService(TcService):

    @classmethod
    def getAdmissibility(cls, operationConfigPerspective: Cfg0ConfiguratorPerspective, admissibilityInputs: List[GetAdmissibilityInputStruct]) -> GetAdmissibilityResponse:
        """
        This operation returns the admissibility statements for the variability data associated with a given set of
        context objects. The instance of Cfg0AbsAdmissibility object represents the relation and state between a
        configurator specific object and a non-configurator specific object. For example, the pair of a partition and
        an family is associated with state "Available". The admissibility states are defined by the LOV
        Cfg0AdmissibilityState. The allowed values for this property are "Available" and "Not Available".
        
        Use cases:
        Consider that following set of the data-
        
        Groups-
        Engine-Box (A Family Group) - It has families "Engine" and "Transmission". 
        1. Engine- Petrol, Diesel
        2. Transmission - Manual, Auto
        Wheel (A Family Group) - It has families "Wheel-drive" and "Suspension". 
        1. Wheel-drive - 2-Wheels, 4-Wheels
        2. Suspension - Full-Thrust, Full-Boom
        For the Engine partition object, the families, Engine &amp; Transmission are "Available" and the families
        Wheel-drive &amp; Suspension are "Not Available".
        The response of the operation for  the Engine partition object will have  the list of  Cfg0AbsAdmissibility
        objects for families Engine and Transmission, 1 for each and those will have admissibility state as "Available"
        and also the Cfg0AbsAdmissibility objects for the families Wheel-drive and Suspension, 1 for each and those
        will have admissibility state as "Not Available".
        
        Exceptions:
        >The operation will return a ServiceException if an unknown error occurs. A possible source for unknown errors
        could be a codeful configurator component customization.
        """
        return cls.execute_soa_method(
            method_name='getAdmissibility',
            library='Configurator',
            service_date='2014_12',
            service_name='ConfiguratorManagement',
            params={'operationConfigPerspective': operationConfigPerspective, 'admissibilityInputs': admissibilityInputs},
            response_cls=GetAdmissibilityResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.DebugMonitor._2014_06.JournalBasedTesting import AuxiliaryInfo
from tcsoa.base import TcService
from tcsoa.gen.Internal.DebugMonitor._2015_10.JournalBasedTesting import TerminateJBTResponse


class JournalBasedTestingService(TcService):

    @classmethod
    def terminateJBT(cls, input: AuxiliaryInfo) -> TerminateJBTResponse:
        """
        terminateJBT marks the end of a journal based test and takes care of all the work that needs to be done when
        the test terminates, such as restoring the database to the initial state. It is called at the end of a test
        recording and test replaying. This operation is used internally by Journal Based Testing, which allows a user
        to record a test in Teamcenter and later on replay the test in a different session.
        
        Use cases:
        Case 1: Restore the database to its initial state
        
        Journal Based Testing is sensitive to database state. It requires the database state at the time of replaying a
        test be identical to the state when the test is recorded. To achieve this requirement, we utilize the
        Teamcenter mark point mechanism. During the test recording, the initializeJBT (an existing operation in the
        same service) is called at the beginning of the session to set the mark point. After the test recording is
        done, terminateJBT will be called to rollback the database to the mark point set by initializeJBT. This will
        ensure that whatever changes made during the test will be undone, and the next test recording will start from
        the same initial DB state.
        
        The database for Journal Based Testing is created using the Teamcenter tcdbinstaller utility. The utility has
        an option to export the DB to a dump file and later on, if the database is corrupted, the DB can be recovered
        from the dump. This dump file captures the initial DB state and so we can use it to recover the DB to the same
        initial state in cases where the client loses server connection or the terminateJBT operation fails to rollback
        the database for whatever reasons. Currently the command to restore the DB from dump is wrapped in a batch
        file, which is executed manually.
        """
        return cls.execute_soa_method(
            method_name='terminateJBT',
            library='Internal-DebugMonitor',
            service_date='2015_10',
            service_name='JournalBasedTesting',
            params={'input': input},
            response_cls=TerminateJBTResponse,
        )



from __future__ import annotations

from tcsoa.gen.Core._2012_09.ProjectLevelSecurity import ProjectTeamsResponse, ProjectClientId, ProjectOpsResponse, CopyProjectsInfo, ProjectInformation, ModifyProjectsInfo
from typing import List
from tcsoa.gen.Core._2012_09.DataManagement import RelateInfoIn
from tcsoa.gen.Core._2011_06.DataManagement import SaveAsIn, SaveAsObjectsResponse
from tcsoa.base import TcService


class ProjectLevelSecurityService(TcService):

    @classmethod
    def modifyProjects(cls, modifyProjectsInfos: List[ModifyProjectsInfo]) -> ProjectOpsResponse:
        """
        This operation modifies the given list of TC_Project objects using the input specified. The input contains new
        values for all the project properties. Values for properties other than the project team are ignored unless the
        user is the Project Administrator.
        
        The entire Project Team, with the exception of the Project Administrator, is replaced with the specified team.
        Therefore, a Project Team Administrator must be specified. If the new Project Team is different than the
        current team, the user performing this operation must be either the Project Administrator or Project Team
        Administrator for the project being modified.
        """
        return cls.execute_soa_method(
            method_name='modifyProjects',
            library='Core',
            service_date='2012_09',
            service_name='ProjectLevelSecurity',
            params={'modifyProjectsInfos': modifyProjectsInfos},
            response_cls=ProjectOpsResponse,
        )

    @classmethod
    def copyProjects(cls, copyProjectsInfos: List[CopyProjectsInfo]) -> ProjectOpsResponse:
        """
        This operation copies  the given list of TC_Project objects. The operation also copies any information which is
        in contained in the project. Data such as project team members and any objects assigned to the source project
        will also be copied to the new project. If a project with given project ID exists in the system then this
        operation will return error 101010.  The operation will continue with copying the other projects.
        """
        return cls.execute_soa_method(
            method_name='copyProjects',
            library='Core',
            service_date='2012_09',
            service_name='ProjectLevelSecurity',
            params={'copyProjectsInfos': copyProjectsInfos},
            response_cls=ProjectOpsResponse,
        )

    @classmethod
    def createProjects(cls, projectInfos: List[ProjectInformation]) -> ProjectOpsResponse:
        """
        This operation creates TC_Project objects using the given input information. If the project with given project
        ID exists in the system then this operation will return unique id violation error 101010.  However, creation of
        rest of the projects will continue.
        """
        return cls.execute_soa_method(
            method_name='createProjects',
            library='Core',
            service_date='2012_09',
            service_name='ProjectLevelSecurity',
            params={'projectInfos': projectInfos},
            response_cls=ProjectOpsResponse,
        )

    @classmethod
    def getProjectTeams(cls, projectObjs: List[ProjectClientId]) -> ProjectTeamsResponse:
        """
        This operation returns team members for the given list of TC_Project objects.
        """
        return cls.execute_soa_method(
            method_name='getProjectTeams',
            library='Core',
            service_date='2012_09',
            service_name='ProjectLevelSecurity',
            params={'projectObjs': projectObjs},
            response_cls=ProjectTeamsResponse,
        )


class DataManagementService(TcService):

    @classmethod
    def saveAsObjectAndRelate(cls, saveAsInput: List[SaveAsIn], relateInfo: List[RelateInfoIn]) -> SaveAsObjectsResponse:
        """
        This operation saves the given object and its related objects as new instances. Related objects are identifed
        using deep copy rules. Optionally,this method relates the new object to the input target object or to a default
        folder.
        """
        return cls.execute_soa_method(
            method_name='saveAsObjectAndRelate',
            library='Core',
            service_date='2012_09',
            service_name='DataManagement',
            params={'saveAsInput': saveAsInput, 'relateInfo': relateInfo},
            response_cls=SaveAsObjectsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Core._2010_04.DataManagement import DatasetOutput
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class CreateAttachOut(TcBaseObj):
    """
    This is output structure for create operation including unique client identifier.
    
    :var clientId: Unique client identifier
    :var objects: List of objects that were created.
    :var datasets: List of output structure for created Dataset objects. Each element in the list contains a client id,
    the related Dataset object created, and the information needed to upload the Dataset object's named reference
    """
    clientId: str = ''
    objects: List[BusinessObject] = ()
    datasets: List[DatasetOutput] = ()


@dataclass
class CreateAttachResponse(TcBaseObj):
    """
    This is response object structure for the createAttachAndSubmitObjects operation
    
    :var output: List of output objects representing objects that were created
    :var serviceData: Service data including partial errors that are mapped to the client id
    """
    output: List[CreateAttachOut] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Manufacturing._2014_12.StructureSearch import ResolveAssignmentRecipeInputElement, CreateOrUpdateAssignmentRecipeResp, ResolveAssignmentRecipeResp, GetAssignmentRecipesResp, CreateOrUpdateAssignmentRecipeInputElem
from tcsoa.gen.Manufacturing._2014_12.IPAManagement import RepopulateDynamicIPAsResponse, RepopulateDynamicIPAsData, FindAndRepopulateDynamicIPAsResponse
from typing import List
from tcsoa.gen.Manufacturing._2014_12.Validation import MaturityReportResponse, MaturityReportRequest
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService
from tcsoa.gen.Manufacturing._2014_12.Model import ValidateScopeFlowsConsistencyResponse


class IPAManagementService(TcService):

    @classmethod
    def repopulateDynamicIPAs(cls, input: RepopulateDynamicIPAsData) -> RepopulateDynamicIPAsResponse:
        """
        The operation converts the given input list of absolute occurrence IDs to a list of objects (ImanItemBOPLine)
        and calculates/regenerates & returns dynamic in-process assembly nodes and its related information. The input
        object contains the top BOPLine and a list of absolute occurrence IDs of processes(Mfg0BvrProcess) or shared
        studies(Mfg0BvrShdStudy).
        
        For every given absolute occurrence IDs
        - If the object doesn't have any dynamic in-process-assembly (IPA)  as children, the operation issues an error.
        - If the dynamic IPA nodes (that belong to the given object) are empty, the operation re-calculates them. 
        - The response of the operation includes the dynamic IPA nodes that belongs to the object. For each dynamic
        IPA, the response includes its content (i.e. parts underneath) and occurrence information. Both the consumed
        part object (BOPLine) and referenced part object (BOMLine) are returned along with their related occurrence
        information. 
        
        
        
        This data is returned for every given object, no matter whether its dynamic IPA nodes were originally empty or
        not.
        """
        return cls.execute_soa_method(
            method_name='repopulateDynamicIPAs',
            library='Manufacturing',
            service_date='2014_12',
            service_name='IPAManagement',
            params={'input': input},
            response_cls=RepopulateDynamicIPAsResponse,
        )

    @classmethod
    def findAndRepopulateDynamicIPAs(cls, inputBOPLines: List[BusinessObject]) -> FindAndRepopulateDynamicIPAsResponse:
        """
        This operation recieves a list of ImanItemBOPLine as an input.
        (The type of of the bop line objects can be either Mfg0BvrProcess or Mfg0BvrShdStudy).
        
        For every given object:
        1. If the object does not have any dynamic IPAs, the operation issues an error.
        
        2. If the dynamic IPAs (that belong to the given object) are empty, the operation re-calculates them.
        """
        return cls.execute_soa_method(
            method_name='findAndRepopulateDynamicIPAs',
            library='Manufacturing',
            service_date='2014_12',
            service_name='IPAManagement',
            params={'inputBOPLines': inputBOPLines},
            response_cls=FindAndRepopulateDynamicIPAsResponse,
        )


class StructureSearchService(TcService):

    @classmethod
    def resolveAssignmentRecipe(cls, input: List[ResolveAssignmentRecipeInputElement]) -> ResolveAssignmentRecipeResp:
        """
        This operation resolves the saved search recipe on a Manufacturing BOM (MBOM) node. The recipe is used to
        automatically resolve Engineering BOM (EBOM) lines under the MBOM node with the recipe. The operation will
        throw an ServiceException if the Teamcenter database schema does not have the recipe constructs.
        
        Exceptions:
        >If the schema for the AssignmentRecipe is not available an exception is thrown.
        """
        return cls.execute_soa_method(
            method_name='resolveAssignmentRecipe',
            library='Manufacturing',
            service_date='2014_12',
            service_name='StructureSearch',
            params={'input': input},
            response_cls=ResolveAssignmentRecipeResp,
        )

    @classmethod
    def createOrUpdateAssignmentRecipe(cls, input: List[CreateOrUpdateAssignmentRecipeInputElem]) -> CreateOrUpdateAssignmentRecipeResp:
        """
        This operation creates or updates the search recipe on a Manufacturing BOM (MBOM) node. The recipe is used to
        automatically resolve Engineering BOM (EBOM) lines under the MBOM node with the recipe. If recipe is updated
        after a prior resolve there must be a subsequent call to resolve the new recipe using the
        resolveAssignmentRecipe. The operation will throw an ServiceException if the Teamcenter database schema does
        not have the recipe constructs. It requires the recipe to be provided as a SearchStructureContext object
        (capturing structure search parameters) and/or key-value pairs of AbsOccurrence attributes.
        
        Use cases:
        There is a need to automatically consume Engineering BOM (EBOM) nodes under a phantom Manufacturing BOM (MBOM)
        node based on search criteria provided by Structure Search.
        
        Exceptions:
        >If the schema for the Mfg0MEMBOMRecipe is not found, an exception will be thrown.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateAssignmentRecipe',
            library='Manufacturing',
            service_date='2014_12',
            service_name='StructureSearch',
            params={'input': input},
            response_cls=CreateOrUpdateAssignmentRecipeResp,
        )

    @classmethod
    def deleteAssignmentRecipes(cls, recipes: List[BusinessObject], recipeAnchors: List[BusinessObject], contextForRemovingResolvedObjs: BusinessObject, appKey: str) -> ServiceData:
        """
        This operation deletes the given explit recipe (Mfg0MEMBOMRecipe) objects. Optionally, can delete all recipe
        objects attached to the recipeAnchors ( BOMLine Objects). If the contextForRemovingResolvedObjs is provided,
        the resolved lines will be removed unless those are resolved by other recipes too. Pass NULL for this parameter
        if resolved lines are not to be cleaned up.
        
        Exceptions:
        >If the schema for the recipe is not installed, service will throw an exception.
        """
        return cls.execute_soa_method(
            method_name='deleteAssignmentRecipes',
            library='Manufacturing',
            service_date='2014_12',
            service_name='StructureSearch',
            params={'recipes': recipes, 'recipeAnchors': recipeAnchors, 'contextForRemovingResolvedObjs': contextForRemovingResolvedObjs, 'appKey': appKey},
            response_cls=ServiceData,
        )

    @classmethod
    def getAssignmentRecipes(cls, recipeAnchors: List[BusinessObject], recipeNames: List[str], appKey: str) -> GetAssignmentRecipesResp:
        """
        This operation get the recipe (Mfg0MEMBOMRecipe) objects attached to the underlying ItemRevisions of the
        recipeAnchors. Currently, only BOMLine objects can be specified as recipeAnchors.
        
        Exceptions:
        >If the schema for Manufacturing BOM (MBOM) does not exist the service exception is thrown.
        """
        return cls.execute_soa_method(
            method_name='getAssignmentRecipes',
            library='Manufacturing',
            service_date='2014_12',
            service_name='StructureSearch',
            params={'recipeAnchors': recipeAnchors, 'recipeNames': recipeNames, 'appKey': appKey},
            response_cls=GetAssignmentRecipesResp,
        )


class ModelService(TcService):

    @classmethod
    def validateScopeFlowsConsistency(cls, rootLines: List[BusinessObject]) -> ValidateScopeFlowsConsistencyResponse:
        """
        This operation will be used to identify scope flows created under the given input line are acyclic or not. This
        operation recieves a list of ImanItemBOPLine as an input.  (The type of the BOP line objects can be either
        Mfg0BvrProcess or Mfg0BvrOperation). For each input scope, the operation checks whether the scope-flows
        (defined in this scope), create a cycle of processes or operations. 
        
        If a cycle is indentified during this operation, the relevant information for the cycle is returned in the
        response.
        """
        return cls.execute_soa_method(
            method_name='validateScopeFlowsConsistency',
            library='Manufacturing',
            service_date='2014_12',
            service_name='Model',
            params={'rootLines': rootLines},
            response_cls=ValidateScopeFlowsConsistencyResponse,
        )


class ValidationService(TcService):

    @classmethod
    def getMaturityReport(cls, maturityReportRequest: MaturityReportRequest) -> MaturityReportResponse:
        """
        This service operation evaluates the maturity of the structure based on certain rules. The operation executes 
        specified rules on objects of the structure and returns if the rules are fulfilled or not. The operation takes 
        BOMLine as a scope to evaluate the maturity of objects under it, the list of rules to evaluate the maturity,
        and supporting information. In response the operation returns the objects and their maturity status along with
        other relevant information.
        
        Use cases:
        Use Case 1 - User checks maturity of a structure.
            User can check the maturity of structure by right clicking on root or topline of the structure. Consider a
        Logistic structure consisting of logistics bill of process (LBOP), part family, parts under the part family and
        in-plant supply chain. User needs to know if all part families have part specified or if all part families have
        at least one in-plant supply chain or if all parts have at least one in-bound supply chain. User can choose the
        predefined rules and use this operation to evaluate the rules on part family and in response see a maturity
        report.
        """
        return cls.execute_soa_method(
            method_name='getMaturityReport',
            library='Manufacturing',
            service_date='2014_12',
            service_name='Validation',
            params={'maturityReportRequest': maturityReportRequest},
            response_cls=MaturityReportResponse,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetSubscribableTypesAndSubtypesResponse(TcBaseObj):
    """
    It returns  'TypesAndSubtypesData'  containing type name and associated display names. If there are any unexpected
    errors while getting the licenses, the errors are returned in 'serviceData' of 
    'GetSubscribableTypesAndSubtypesResponse'. Partial errors are returned via sericeData if there are any.
    
    :var serviceData: Teamcenter service response data
    :var typesNames: structure containing typeNames and associated Display names.
    """
    serviceData: ServiceData = None
    typesNames: List[TypesAndSubtypesData] = ()


@dataclass
class TypesAndSubtypesData(TcBaseObj):
    """
    'TypesAndSubtypesData' structure represents 'typeName' and associated 'displayName' for a subscribable types and
    subtypes.
    
    :var displayName: Contains display name associated with the typename.
    :var typeName: Contains type name for a type and subtype that is subscribable.
    """
    displayName: str = ''
    typeName: str = ''



from __future__ import annotations

from tcsoa.gen.Internal.Rdv._2010_04.VariantManagement import AddPartSolutionResponse, TargetOccurrences, ReplacePartSolutionResponse, BackgroundOccurrencesResponse, ReplacePartSolutionInputInfo, AddPartSolutionInputInfo
from typing import List
from tcsoa.base import TcService


class VariantManagementService(TcService):

    @classmethod
    def getValidBackgroundOverlays(cls, abortOnError: bool, questions: List[TargetOccurrences]) -> BackgroundOccurrencesResponse:
        """
        This method returns valid background (MEAppearancePathNode or Absocc) UIDs for the respective target (APN or
        Absocc) UID and proximity. Given the top level ItemRevision object tag, RevisionRule name and list of 1:1
        target UIDs and proximity, list of valid background overlays is returned. Proximity search and Valid Overlays
        Only (VOO) filtering is applied on the APN target UIDs to obtain the valid APN background overlays. 
        Please note that the target UIDs provided could be APN or Absocc. If target UID are Absocc, then it is
        converted to APN UID before doing proximity search and Valid Overlay Only processing. Resulting APN background
        overlays are converted to Absocc before being sent to client, if the inputs were Absocc UIDs.
        If the 'abortOnError' flag is set to true, the method would abort even if it finds a single target UID that it
        cannot process. If it is set to false, it would process all the UIDs and return the respective failure codes to
        the client.
        
        Use cases:
        Use case 1: Perform Proximity Search and Valid Overlays Only filtering, for the target MEAPN UIDs, based on the
        proximity distance provided for each of them.
        
        Use case 2: Perform Proximity Search and Valid Overlays Only filtering, for the target Absocc UIDs, based on
        the proximity distance provided for each of them.
        """
        return cls.execute_soa_method(
            method_name='getValidBackgroundOverlays',
            library='Internal-Rdv',
            service_date='2010_04',
            service_name='VariantManagement',
            params={'abortOnError': abortOnError, 'questions': questions},
            response_cls=BackgroundOccurrencesResponse,
        )

    @classmethod
    def replacePartInProduct(cls, inputs: List[ReplacePartSolutionInputInfo]) -> ReplacePartSolutionResponse:
        """
        Replaces a list of existing Part Solutions inside a Product specific Architecture Breakdown Structure with new
        Part Solutions. It also replaces the existing associated part data (Named Variant Expressions and Occurrence
        Notes) corresponding to each part solution with new part data. The new Part solutions and the part data is
        specified through the 'ReplacePartSolutionInputInfo' object. The Part Solution can be replaced with itself (for
        updating) or with a new Part Solution.
        
        Use cases:
        The 'replacePartInProduct' operation is called when user wants to replace existing part solutions and its
        corresponding part data on an Architecture Breakdown Element with new Part Solutions and part data.
        """
        return cls.execute_soa_method(
            method_name='replacePartInProduct',
            library='Internal-Rdv',
            service_date='2010_04',
            service_name='VariantManagement',
            params={'inputs': inputs},
            response_cls=ReplacePartSolutionResponse,
        )

    @classmethod
    def addPartToProduct(cls, inputs: List[AddPartSolutionInputInfo]) -> AddPartSolutionResponse:
        """
        Adds a list of Part Solutions (ERP parts) to an Architecture Breakdown Element (ABE) inside a Product specific
        Architecture Breakdown structure. It also associates the part data (Named Variant Expressions and Occurrence
        Notes) corresponding to each part solution to the architecture breakdown element. The input part solution and
        the part data is specified through the 'AddPartSolutionInputInfo' object.
        
        Use cases:
        The 'addPartToProduct' operation is called when user wants to add part solutions and associate the
        corresponding part data to an Architecture Breakdown Element. The user can specify the input part solution and
        the part data using 'AddPartSolutionInputInfo' object.
        """
        return cls.execute_soa_method(
            method_name='addPartToProduct',
            library='Internal-Rdv',
            service_date='2010_04',
            service_name='VariantManagement',
            params={'inputs': inputs},
            response_cls=AddPartSolutionResponse,
        )



from tcsoa.gen.Internal_Manufacturing._2017_11.services import ResourceManagementService as imp0
from tcsoa.gen.Internal_Manufacturing._2015_03.services import StructureManagementService as imp1
from tcsoa.gen.Internal_Manufacturing._2015_10.services import StructureManagementService as imp2
from tcsoa.gen.Internal_Manufacturing._2020_01.services import DataManagementService as imp3
from tcsoa.gen.Internal_Manufacturing._2018_11.services import ResourceManagementService as imp4
from tcsoa.gen.Internal_Manufacturing._2014_06.services import DataManagementService as imp5
from tcsoa.gen.Internal_Manufacturing._2008_12.services import CoreService as imp6
from tcsoa.gen.Internal_Manufacturing._2014_06.services import IPAManagementService as imp7
from tcsoa.gen.Internal_Manufacturing._2016_09.services import IPAManagementService as imp8
from tcsoa.gen.Internal_Manufacturing._2016_09.services import StructureManagementService as imp9
from tcsoa.gen.Internal_Manufacturing._2017_11.services import StructureManagementService as imp10
from tcsoa.gen.Internal_Manufacturing._2015_03.services import AttachmentsService as imp11
from tcsoa.gen.Internal_Manufacturing._2016_09.services import DataManagementService as imp12
from tcsoa.gen.Internal_Manufacturing._2016_03.services import ResourceManagementService as imp13
from tcsoa.gen.Internal_Manufacturing._2014_12.services import ModelService as imp14
from tcsoa.gen.Internal_Manufacturing._2018_11.services import StructureManagementService as imp15
from tcsoa.gen.Internal_Manufacturing._2017_05.services import ImportExportService as imp16
from tcsoa.gen.Internal_Manufacturing._2016_09.services import ResourceManagementService as imp17
from tcsoa.gen.Internal_Manufacturing._2015_10.services import StructureSearchService as imp18
from tcsoa.gen.Internal_Manufacturing._2016_09.services import StructureSearchService as imp19
from tcsoa.gen.Internal_Manufacturing._2019_06.services import ClosureRuleEditorService as imp20
from tcsoa.gen.Internal_Manufacturing._2017_05.services import DataManagementService as imp21
from tcsoa.gen.Internal_Manufacturing._2019_06.services import ResourceManagementService as imp22
from tcsoa.gen.Internal_Manufacturing._2020_04.services import DataManagementService as imp23
from tcsoa.gen.Internal_Manufacturing._2015_03.services import StructureSearchService as imp24
from tcsoa.gen.Internal_Manufacturing._2014_12.services import ResourceManagementService as imp25
from tcsoa.gen.Internal_Manufacturing._2015_03.services import ResourceManagementService as imp26
from tcsoa.gen.Internal_Manufacturing._2012_09.services import StructureSearchService as imp27
from tcsoa.gen.Internal_Manufacturing._2011_12.services import DataManagementService as imp28
from tcsoa.gen.Internal_Manufacturing._2014_12.services import IPAManagementService as imp29
from tcsoa.gen.Internal_Manufacturing._2019_06.services import StructureSearchService as imp30
from tcsoa.gen.Internal_Manufacturing._2020_01.services import StructureSearchService as imp31
from tcsoa.gen.Internal_Manufacturing._2013_12.services import ResourceManagementService as imp32
from tcsoa.gen.Internal_Manufacturing._2017_05.services import StructureManagementService as imp33
from tcsoa.base import TcService


class ResourceManagementService(TcService):
    addMultiToolCutter = imp0.addMultiToolCutter
    autoPositionComponentByCSYS = imp4.autoPositionComponentByCSYS
    createSetupSheets = imp13.createSetupSheets
    deleteMultiToolCutter = imp0.deleteMultiToolCutter
    extractHolderData = imp17.extractHolderData
    getICOMappingTargets = imp22.getICOMappingTargets
    getStepP21FileCounts2 = imp25.getStepP21FileCounts2
    getVendorCatalogInfo2 = imp25.getVendorCatalogInfo2
    getVendorCatalogInfo3 = imp17.getVendorCatalogInfo3
    importStep3DModels2 = imp26.importStep3DModels2
    importStepP21Files2 = imp25.importStepP21Files2
    importStepP21Files3 = imp26.importStepP21Files3
    mapClassificationObject = imp26.mapClassificationObject
    showCoordinateSystems = imp32.showCoordinateSystems
    showGCSConnectionPoints = imp32.showGCSConnectionPoints
    unzipGtcPackage = imp26.unzipGtcPackage


class StructureManagementService(TcService):
    alignAssemblies = imp1.alignAssemblies
    alignLinesInBOM = imp2.alignLinesInBOM
    completenessCheckPartStructure = imp9.completenessCheckPartStructure
    configureMultipleStructures = imp1.configureMultipleStructures
    createAlternativeScopeForProduct = imp10.createAlternativeScopeForProduct
    createAsyncCollabPlanningContext = imp9.createAsyncCollabPlanningContext
    createOrUpdateDesignPartAlignment = imp9.createOrUpdateDesignPartAlignment
    createReuseAssemblies = imp1.createReuseAssemblies
    evaluateLinks = imp15.evaluateLinks
    findBrokenPartsInProductView = imp15.findBrokenPartsInProductView
    findBrokenProductViews = imp15.findBrokenProductViews
    findRelatedDesignOrPartStructures = imp9.findRelatedDesignOrPartStructures
    getClusterDetails = imp1.getClusterDetails
    getEquivalentPropertyValues = imp2.getEquivalentPropertyValues
    linkOrUnlinkStructures = imp9.linkOrUnlinkStructures
    pasteByRule = imp10.pasteByRule
    pasteOrReplaceAssemblyInContext = imp2.pasteOrReplaceAssemblyInContext
    removeDesignPartAlignment = imp9.removeDesignPartAlignment
    searchForClusters = imp1.searchForClusters
    syncMasterAndAlternative = imp33.syncMasterAndAlternative
    verifyDesignPartAlignment = imp9.verifyDesignPartAlignment


class DataManagementService(TcService):
    associateOrRemoveScopesForProcess = imp3.associateOrRemoveScopesForProcess
    automaticMFGFeaturesAssignment = imp5.automaticMFGFeaturesAssignment
    createBOEfromPlantBOP = imp12.createBOEfromPlantBOP
    getFutureRevisions = imp21.getFutureRevisions
    getProductScopeForProcess = imp23.getProductScopeForProcess
    linkPlantBOPtoBOE = imp12.linkPlantBOPtoBOE
    postAssignIDICMaker = imp28.postAssignIDICMaker
    synchronizePlantBOPAndBOE = imp12.synchronizePlantBOPAndBOE


class CoreService(TcService):
    checkinForProcessSimulate = imp6.checkinForProcessSimulate
    checkoutForProcessSimulate = imp6.checkoutForProcessSimulate


class IPAManagementService(TcService):
    cleanDynamicIPALines = imp7.cleanDynamicIPALines
    cleanDynamicIPALines2 = imp8.cleanDynamicIPALines
    createDynamicIPALines = imp8.createDynamicIPALines
    getDynamicIPALines = imp7.getDynamicIPALines
    saveContentOfDynamicIPALines = imp8.saveContentOfDynamicIPALines
    searchDynamicIPAs = imp29.searchDynamicIPAs
    updateDynamicIPALines = imp8.updateDynamicIPALines


class AttachmentsService(TcService):
    createAttachmentLines = imp11.createAttachmentLines
    getAttachmentLines = imp11.getAttachmentLines


class ModelService(TcService):
    deleteScopeFlows = imp14.deleteScopeFlows
    getUILocations = imp14.getUILocations
    laAsyncResolve = imp14.laAsyncResolve
    saveUILocations = imp14.saveUILocations
    scheduleLAResolve = imp14.scheduleLAResolve


class ImportExportService(TcService):
    exportToBriefcaseAsync = imp16.exportToBriefcaseAsync
    importFromBriefcaseAsync = imp16.importFromBriefcaseAsync


class StructureSearchService(TcService):
    findCollabPlanningContext = imp18.findCollabPlanningContext
    findEquivalentLines = imp18.findEquivalentLines
    getBOEForPlantBOPScope = imp19.getBOEForPlantBOPScope
    getSearchCriteriaFromRecipe = imp24.getSearchCriteriaFromRecipe
    mapSrchCriteriaToLines = imp27.mapSrchCriteriaToLines
    saveOGLinesInSrchCriteria = imp27.saveOGLinesInSrchCriteria
    searchPartsInProximityOfMFGFeatures = imp30.searchPartsInProximityOfMFGFeatures
    searchScopedStructure = imp31.searchScopedStructure


class ClosureRuleEditorService(TcService):
    getClosureRuleTraversalInfo = imp20.getClosureRuleTraversalInfo
    setClosureRuleTraversalInfo = imp20.setClosureRuleTraversalInfo



from tcsoa.gen.CalendarManagement._2007_06.services import CalendarManagementService as imp0
from tcsoa.gen.CalendarManagement._2008_06.services import CalendarManagementService as imp1
from tcsoa.gen.CalendarManagement._2007_01.services import CalendarManagementService as imp2
from tcsoa.base import TcService


class CalendarManagementService(TcService):
    createCalendars = imp0.createCalendars
    createCalendars2 = imp1.createCalendars
    createTCCalendars = imp2.createTCCalendars
    deleteTCCalendars = imp2.deleteTCCalendars
    getTCCalendars = imp2.getTCCalendars
    modifyTCCalendars = imp2.modifyTCCalendars
    updateCalendars = imp0.updateCalendars
    updateCalendars2 = imp1.updateCalendars
    updateTCCalendars = imp2.updateTCCalendars



from __future__ import annotations

from tcsoa.gen.Internal.Query._2012_02.SavedQuery import DescribeSavedQueryDefinitionInput
from tcsoa.gen.Internal.Query._2014_10.SavedQuery import DescribeSavedQueryDefnResponse3
from typing import List
from tcsoa.base import TcService


class SavedQueryService(TcService):

    @classmethod
    def describeSavedQueryDefinitions3(cls, requestedQueries: List[DescribeSavedQueryDefinitionInput]) -> DescribeSavedQueryDefnResponse3:
        """
        This operation returns detailed information about a saved query, including the search criteria and default sort
        attributes.  The search criteria is made up of a list of attributes and the parameters used to evaluate them. 
        End user running the query will be able to enter a search value for each attribute listed in the search
        criteria.  The criteria will indicate the math operation ("!=", ">", etc) to use when evaluating the users
        input.  Default values are also included.
        The sort attributes are a way to define a default sorting to use whenever the saved query is executed.  For
        each attribute, an "Ascending" or "Descending" sort order will be included.  Any user running the saved query
        will see the results sorted according to the sorting defined on the saved query.  The end user can use the Sort
        dialog to override what is defined.
        All of the attributes in both lists will include the L10N key to be used when displaying the attribute to the
        user. Also all the attributes will be indicated whether a LOVs are attached to them or not. Use this
        information to render LOV UI for the attributes. This requires LOV service operations which can used to
        retrieve the LOV values to display to the end user on request. 
        """
        return cls.execute_soa_method(
            method_name='describeSavedQueryDefinitions3',
            library='Internal-Query',
            service_date='2014_10',
            service_name='SavedQuery',
            params={'requestedQueries': requestedQueries},
            response_cls=DescribeSavedQueryDefnResponse3,
        )



from __future__ import annotations

from tcsoa.base import TcService


class SessionService(TcService):

    @classmethod
    def getSecurityToken(cls, duration: int) -> str:
        """
        This operation returns a security token for the current user. This token can be used as the 'password' argument
        on the 'login' service operations. The token is valid for the duration specified on the input, if used in the
        'login' operation after that duration, the 'login' operation will fail. A session established with this token
        is still subject to a session time-out due to inactivity. At which point the client application will be
        presented with an authentication challenge through the 'CredentialManager' interface. The client application's
        implementation of the 'CredentialManager' interface is responsible for obtaining the user's credentials (i.e.
        prompting the user) before continuing on with the session.
        
        Exceptions:
        >- 214184:    Failed to generate the security token.
        
        """
        return cls.execute_soa_method(
            method_name='getSecurityToken',
            library='Internal-Core',
            service_date='2014_11',
            service_name='Session',
            params={'duration': duration},
            response_cls=str,
        )



from __future__ import annotations

from typing import List
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class LicenseDetails(TcBaseObj):
    """
    LicenseDetails structure represents all the details of an ADA License object.
    
    :var licenseType: The type of ADA License object. Supported types are 
    &amp;lt;i&amp;gt;ITAR_License&amp;lt;/i&amp;gt;, &amp;lt;i&amp;gt;Exclude_License&amp;lt;/i&amp;gt;, or
    &amp;lt;i&amp;gt;IP_License&amp;lt;/i&amp;gt;.
    :var licenseId: The unique ID of the license. This is string with a maximum of 128 characters.
    :var groups: A list of names of groups associated with the license identified by &amp;lt;font
    face=&amp;amp;quot;courier&amp;amp;quot;
    height=&amp;amp;quot;10&amp;amp;quot;&amp;gt;licenseId&amp;lt;/font&amp;gt;. For subgroups, the full names should
    be specified. When the license is attached to a classified WorkspaceObject;, the users from the groups or subgroups
    listed on the license will get access to the workspace objects, based on the access rules. This parameter
    represents an array of group name strings of upto 256 characters in size.
    :var newLicenseId: The unique ID of the license. This is string with a maximum of 128 characters. A non-empty value
    indicates the licenseId will be replaced by newLicenseId.
    :var licenseCategory: The category property of an ADA license. This is an optional parameter of string type 128
    characters in size. The value specified for licenseCategory parameter must match one of the values in the LOV
    associated with Category property on the specifc ADA license type.This is an optional parameter and may be left as
    a blank string (&amp;quot;&amp;quot;).
    :var userCitizenships: A list of user citizenships property values of an ADA license. This is an optional
    parameter. The value specified for userCitizenships parameter is an ISO-3166 two-letter country code. If a valid
    country code is not specified, partial error code 10219 will be returned.
    :var expiryDate: The expiry date for the license, after which the license cannot be attached to a WorkspaceObject
    and ceases to grant the access to users or groups listed on the license. The value specified for expiryDate should
    be greater than or equal to current date, unless the value is same as current value on an existing license. A NULL
    date specifies the license will never expire.
    :var lockDate: The freeze date for the license, after which the license cannot be attached to a WorkspaceObject.
    The value specified for this parameter should be greater than the current date and should be less than the value
    specified for expiryDate parameter, unless the value is same as current value on an existing license. A NULL date
    specifies the license is not locked.
    :var licenseReason: A reason for the license to exist. This parameter can be a maximum of 128 characters. This is
    an optional parameter and and  may be left as a blank string (&amp;quot;&amp;quot;).
    :var qualifyingCfr: The qualifying Code of Federal Regulations (CFR) for ITAR licenses. This is not applicable for
    IP and Exclude licenses. This parameter can be a maxiumum of 80 characters. This is an optional parameter and  may
    have value as a blank string (&amp;quot;&amp;quot;).
    :var users: A list of users associated with a license identified by &amp;lt;font
    face=&amp;amp;quot;courier&amp;amp;quot;
    height=&amp;amp;quot;10&amp;amp;quot;&amp;gt;licenseId&amp;lt;/font&amp;gt;. When the license is attached to a
    classified WorkspaceObject, the users listed on the license will get access to the WorkspaceObject, based on the
    access rules.
    """
    licenseType: str = ''
    licenseId: str = ''
    groups: List[str] = ()
    newLicenseId: str = ''
    licenseCategory: str = ''
    userCitizenships: List[str] = ()
    expiryDate: datetime = None
    lockDate: datetime = None
    licenseReason: str = ''
    qualifyingCfr: str = ''
    users: List[str] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, WorkspaceObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetManagedRelationInput(TcBaseObj):
    """
    GetManagedRelationInput
    
    :var primaryTags: List of primaryTags
    :var secondaryTags: List of secondaryTags
    :var primaryType: // primaryType of Managed relation
    :var subtype: // subtype of primary type
    """
    primaryTags: List[BusinessObject] = ()
    secondaryTags: List[BusinessObject] = ()
    primaryType: str = ''
    subtype: str = ''


@dataclass
class GetManagedRelationResponse(TcBaseObj):
    """
    GetManagedRelation Response
    
    :var managedRelations: Tracelink relations
    :var serviceData: The successful Object ids, partial errors and failures
    """
    managedRelations: List[WorkspaceObject] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Internal.ImportExport._2010_04.L10NImportExport import ExportObjectsForTranslationResponse
from tcsoa.gen.Internal.ImportExport._2018_06.L10NImportExport import ObjectsInfo2
from tcsoa.base import TcService


class L10NImportExportService(TcService):

    @classmethod
    def exportObjectsForTranslation2(cls, inputObjectsInfo: ObjectsInfo2) -> ExportObjectsForTranslationResponse:
        """
        This operation exports Teamcenter business objects into XML file based on the criteria specified by
        'ObjectsInfo2' structure. This operation is used to export Teamcenter business object that contains localized
        properties so that site administrator or other privileged user can translate the properties to desired locale
        and import back into database using the
        'Teamcenter::Soa::Internal::ImportExport::_2010_04::L10NImportExport::importObjectsForTranslation' operation.
        If the session user does not have the necessary privilege or if 'ObjectsInfo2' structure is not valid then
        operation would fail and the error is returned in the 'ServiceData'. This operation is used to export selected
        business objects for review or translate into desired locale. These two modes are explained below:
        - Review: This exports business objects into XML file for selected statuses where valid localized translation
        exist for given locale. This exported file can be used by language expert to review existing translation for
        their accuracy.
        - Translate: The operation exports business objects into XML file for which there is no translation so that
        language expert can use this XML file and provide a valid language translation in selected locale.
        
        
        
        Use cases:
        General description covers this section.
        """
        return cls.execute_soa_method(
            method_name='exportObjectsForTranslation2',
            library='Internal-ImportExport',
            service_date='2018_06',
            service_name='L10NImportExport',
            params={'inputObjectsInfo': inputObjectsInfo},
            response_cls=ExportObjectsForTranslationResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Dataset
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetAvailableBusinessObjectTypesResponse(TcBaseObj):
    """
    Returned service response structure to represents the displayable Business Objects information.
    
    :var inputClassToTypes: A map of Business Object names and associated descendant Business Objects (string,
    vector<AvailableBusinessObjectTypeInfo>>)
    :var serviceData: The ServiceData.
    """
    inputClassToTypes: BusinessObjectClassToTypesMap = None
    serviceData: ServiceData = None


@dataclass
class GetDatasetCreationRelatedInfoResponse2(TcBaseObj):
    """
    Holds the Response data from getDatasetCreationRelatedInfo2
    
    :var toolNames: List of Tool names
    :var toolDisplayNames: List of Tool display names.
    :var newDatasetName: The name of the new Dataset, can be an empty string
    :var nameCanBeModified: If true, the name of the Dataset can be modified
    :var initValuePropertyRules: List of properties have the initialized values from property constant attachments
    :var serviceData: Standard 'ServiceData' member
    """
    toolNames: List[str] = ()
    toolDisplayNames: List[str] = ()
    newDatasetName: str = ''
    nameCanBeModified: bool = False
    initValuePropertyRules: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class LocalizableResults(TcBaseObj):
    """
    This data structure contains property names and boolean value to indicate whether this property is localizable.
    
    :var propName: Internal property name.
    :var islocalizable: boolean value to indicate localizability of this property.
    """
    propName: str = ''
    islocalizable: bool = False


@dataclass
class LocalizableStatusInput(TcBaseObj):
    """
    This data structure contains the type of the business object and its internal property names.
    
    :var objTypeName: String Object Type Name
    :var propNames: A list of the internal property names of the object.
    """
    objTypeName: str = ''
    propNames: List[str] = ()


@dataclass
class LocalizableStatusOutput(TcBaseObj):
    """
    This structure contains object type name and localizable information.
    
    :var objTypeName: Object Type Name
    :var results: Result containing localizable information.
    """
    objTypeName: str = ''
    results: List[LocalizableResults] = ()


@dataclass
class LocalizableStatusResponse(TcBaseObj):
    """
    This data structure contains a list of localized results and 'ServiceData'.
    
    :var results: A list of 'LocalizableResults' structure to hold the localizable information for all properties.
    :var serviceData: 'serviceData'
    """
    results: List[LocalizableStatusOutput] = ()
    serviceData: ServiceData = None


@dataclass
class LocalizedPropertyValuesInfo(TcBaseObj):
    """
    This data structure contains business object tag and a list of of 'NameValueLocaleStruct'
    
    :var object: The business object.
    :var propertyValues:  A list of 'NameValueLocaleStruct' that holds property name, value and locale information.
    """
    object: BusinessObject = None
    propertyValues: List[NameValueLocaleStruct] = ()


@dataclass
class LocalizedPropertyValuesList(TcBaseObj):
    """
    This structure contains  a list of output localized property value info and  partial error.
    
    :var output: A list of structure LocalizedPropertyValuesInfo to keep the localized property values info.
    :var partialErrors: Used for storing partial error and standard service data.
    """
    output: List[LocalizedPropertyValuesInfo] = ()
    partialErrors: ServiceData = None


@dataclass
class NameLocaleStruct(TcBaseObj):
    """
    This structure contains a property name and list of possible locales.
    
    :var name: A property name string
    :var locales:  A list of language locales.
    """
    name: str = ''
    locales: List[str] = ()


@dataclass
class NameValueLocaleStruct(TcBaseObj):
    """
    This structure contains localization related information for property values.
    
    :var name: Property name (internal)
    :var values: A list of property values
    :var locale: The name of the locale
    :var seqNum: Sequence number
    :var status: The name of the localization status.
    The status must be one of the following values:
    - for the approved status: "A", "Approved" or the version of the "Approved" string for the client/server log-in
    locale.
    - for the in-review status: "R", "In Review", "In-Review", "InReview" or the version of the "In Review" string for
    the client/server log-in locale.
    - for the pending status: "P", "Pending" or the version of the "Pending" string for the client/server log-in locale.
    - for the invalid status: "I", "Invalid" or the version of the "Invalid" string for the client/server log-in locale.
    - for the master status: "M", "Master" or the version of the "Master" string for the client/server log-in locale.
    
    
    :var statusDesc: Description of statuses used for tooltip on the user interface
    :var master: Master value indication
    """
    name: str = ''
    values: List[str] = ()
    locale: str = ''
    seqNum: int = 0
    status: List[str] = ()
    statusDesc: List[str] = ()
    master: bool = False


@dataclass
class NamedReferenceObjectInfo(TcBaseObj):
    """
    This structure contains information regarding named reference type value, object reference, object type name and
    list of attribute information to set on the object.
    
    :var clientId: Identifier defined by user to track the related object.
    :var object: Object reference of the object to attach to dataset as named reference.
    :var namedReferenceName: The Named Reference from the dataset to this object, required
    :var referenceType: Reference Type. It is either AE_PART_OF or AE_ASSOCIATION
    """
    clientId: str = ''
    object: BusinessObject = None
    namedReferenceName: str = ''
    referenceType: str = ''


@dataclass
class AttributeInfo(TcBaseObj):
    """
    This structure contains the name value pairs.
    
    :var name: Name of the attribute that needs to be set.
    :var values: Values to be set
    """
    name: str = ''
    values: List[str] = ()


@dataclass
class AvailableBusinessObjectTypeInfo(TcBaseObj):
    """
    This structure represents Business Object name, display name, and its hierarchy.
    
    :var type: Name of the Business Object
    :var displayType: Display name of the Business Object
    :var hierarchies: Bottom up hierarchy of the Business Object going up to the input Business Object
    """
    type: str = ''
    displayType: str = ''
    hierarchies: List[str] = ()


@dataclass
class PropertyInfo(TcBaseObj):
    """
    This data structure contains business object and a list of properties and locales.
    
    :var object: The desired business object to retrieved localized property values
    :var propsToget:  A list of NameLocaleStruct to hold the desired properties and locales.
    """
    object: BusinessObject = None
    propsToget: List[NameLocaleStruct] = ()


@dataclass
class BusinessObjectHierarchy(TcBaseObj):
    """
    This structure contains information about a bottom up hierarchy starting with a Business Object name and going up
    the hierarchy of parents up to the primary Business Object.
    
    :var boName: Name of the Business Object
    :var boDisplayName: Display Name of the Business Object
    :var boParents: Names of Business Objects going up the hierarchy of parents up to the primary Business Object
    """
    boName: str = ''
    boDisplayName: str = ''
    boParents: List[str] = ()


@dataclass
class CommitDatasetFileInfo(TcBaseObj):
    """
    This structure contains the basic info for a file to be uploaded to a dataset.
    
    :var dataset: Dataset object reference.
    :var createNewVersion: Flag to create new version ( TRUE ) or not ( FALSE ).
    :var datasetFileTicketInfos: A list of 'DatasetFileTicketInfos'.
    """
    dataset: Dataset = None
    createNewVersion: bool = False
    datasetFileTicketInfos: List[DatasetFileTicketInfo] = ()


@dataclass
class CreateDatasetsResponse(TcBaseObj):
    """
    Return structure for createDatasets operation
    
    :var datasetOutput: List of output structure for creatDatasets operation. Each element in the list contains a
    client Id and the related Dataset object created.
    :var servData: Standard 'ServiceData' member
    """
    datasetOutput: List[DatasetOutput] = ()
    servData: ServiceData = None


@dataclass
class DatasetFileInfo(TcBaseObj):
    """
    Holds the basic info for a file to be uploaded to a Dataset.
    
    :var clientId: A unique string used to identify return data elements and partial errors associated with this input
    structure.
    :var fileName: Name of file to be uploaded.  Filename only, should not contain path to filename.
    :var namedReferenceName: Named Reference relation to file.
    :var isText: Flag to indicate if file is text ( TRUE ) or binary (FALSE ).
    :var allowReplace: Flag to indicate if file can be overwritten ( TRUE ) or not ( FALSE ).
    """
    clientId: str = ''
    fileName: str = ''
    namedReferenceName: str = ''
    isText: bool = False
    allowReplace: bool = False


@dataclass
class DatasetFileTicketInfo(TcBaseObj):
    """
    This structure contains the basic info for a file to be uploaded to a dataset.
    
    :var datasetFileInfo: Member of type DatasetFileInfo.
    :var ticket: ID of ticket.
    """
    datasetFileInfo: DatasetFileInfo = None
    ticket: str = ''


@dataclass
class DatasetInfo(TcBaseObj):
    """
    The DatasetInfo struct represents all of the data necessary to construct the Dataset object.  The basic attributes
    that required are passed as named elements in the struct.  All other attributes are passed as name/value pairs in
    the AttributeInfo struct.  The nrObjectInfos field allows for the attachment of an object that will be related to
    this newly created Dataset.
    
    :var clientId: A unique string used to identify return data elements and partial errors associated with this input
    structure.
    :var name: Name attribute value
    :var container: Object reference of the container used to hold the created Dataset
    :var relationType: Name of the relation type for the Dataset to be created
    :var description: Description attribute value
    :var type: Type attribute value
    :var datasetId: ID attribute value
    :var datasetRev: Revision attribute value
    :var toolUsed: Name of the Tool used to open the created Dataset, may be an empty string.
    :var attrs: List of attribute name values pairs to be set
    :var datasetFileInfos: List of  info of the files to be uploaded
    :var nrObjectInfos: list of info of named references of the Dataset
    """
    clientId: str = ''
    name: str = ''
    container: BusinessObject = None
    relationType: str = ''
    description: str = ''
    type: str = ''
    datasetId: str = ''
    datasetRev: str = ''
    toolUsed: str = ''
    attrs: List[AttributeInfo] = ()
    datasetFileInfos: List[DatasetFileInfo] = ()
    nrObjectInfos: List[NamedReferenceObjectInfo] = ()


@dataclass
class DatasetOutput(TcBaseObj):
    """
    This structure contains return data from 'createDatasets' operation.
    
    :var clientId: Identifier defined by user to track the input criteria.
    :var dataset: Dataset object reference of the created dataset
    :var commitInfo: List of 'CommitDatasetFileInfos'
    """
    clientId: str = ''
    dataset: Dataset = None
    commitInfo: List[CommitDatasetFileInfo] = ()


@dataclass
class DisplayableBusinessObjectsOut(TcBaseObj):
    """
    This structure contains a list of displayable business objects (BO) under a given BO(displayable sub-BO hierarchy).
    
    :var boTypeName: The Business Object name for which displayable Hierarchy is returned.
    :var displayableBOTypeNames: Displayable BO hierarchy
    """
    boTypeName: str = ''
    displayableBOTypeNames: List[BusinessObjectHierarchy] = ()


@dataclass
class DisplayableSubBusinessObjectsResponse(TcBaseObj):
    """
    Structure to hold list of Business Objects and its displayable names.
    
    :var output: List of displayable Business Objects.
    :var serviceData: The Service data.
    """
    output: List[DisplayableBusinessObjectsOut] = ()
    serviceData: ServiceData = None


"""
A map of classes and their types
"""
BusinessObjectClassToTypesMap = Dict[str, List[AvailableBusinessObjectTypeInfo]]



from __future__ import annotations

from tcsoa.gen.StructureManagement._2008_12.StructureVerification import AccCheckInput, AccountabilityCheckResponse
from tcsoa.base import TcService


class StructureVerificationService(TcService):

    @classmethod
    def accountabilityCheck(cls, input: AccCheckInput) -> AccountabilityCheckResponse:
        """
        The operation will call the existing accountability check functions,  which will generate a check result for
        report in the colored display.
        """
        return cls.execute_soa_method(
            method_name='accountabilityCheck',
            library='StructureManagement',
            service_date='2008_12',
            service_name='StructureVerification',
            params={'input': input},
            response_cls=AccountabilityCheckResponse,
        )



from __future__ import annotations

from tcsoa.gen.CalendarManagement._2007_01.CalendarManagement import TCCalendarModification, TCCalendarResponse, TCCalendarUpdate, NewTCCalendar, GetTCCalendarData
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.gen.BusinessObjects import TCCalendar
from tcsoa.base import TcService


class CalendarManagementService(TcService):

    @classmethod
    def getTCCalendars(cls, getTCCalendars: List[GetTCCalendarData]) -> TCCalendarResponse:
        """
        This operation gets the list of  calendars based on the user's request to the application interface. Multiple
        calendars can be obtained through this operation.
        The information needed to get TCCalendar  objects is specified in the 'GetTCCalendarData'  structure. It
        returns 'TCCalendarResponse' which contains response data from the 'getcalendar' request. Errors will be
        returned in the list of partial errors in the 'ServiceData'. 
        """
        return cls.execute_soa_method(
            method_name='getTCCalendars',
            library='CalendarManagement',
            service_date='2007_01',
            service_name='CalendarManagement',
            params={'getTCCalendars': getTCCalendars},
            response_cls=TCCalendarResponse,
        )

    @classmethod
    def modifyTCCalendars(cls, modifications: List[TCCalendarModification]) -> TCCalendarResponse:
        """
        Modifies the specified schedule calendars based on the action string.
        Reset: Reset the schedule calendar to the base calendar
        Merge: Merge the schedule calendar with the base calendar
        """
        return cls.execute_soa_method(
            method_name='modifyTCCalendars',
            library='CalendarManagement',
            service_date='2007_01',
            service_name='CalendarManagement',
            params={'modifications': modifications},
            response_cls=TCCalendarResponse,
        )

    @classmethod
    def updateTCCalendars(cls, tccalendarUpdates: List[TCCalendarUpdate]) -> TCCalendarResponse:
        """
        Updates the specified calendars and their events.
        This includes creating, modifying and deleting events associated with specified calendars
        """
        return cls.execute_soa_method(
            method_name='updateTCCalendars',
            library='CalendarManagement',
            service_date='2007_01',
            service_name='CalendarManagement',
            params={'tccalendarUpdates': tccalendarUpdates},
            response_cls=TCCalendarResponse,
        )

    @classmethod
    def createTCCalendars(cls, newTCCalendars: List[NewTCCalendar]) -> TCCalendarResponse:
        """
        Creates new calendars based on the input parameters
        """
        return cls.execute_soa_method(
            method_name='createTCCalendars',
            library='CalendarManagement',
            service_date='2007_01',
            service_name='CalendarManagement',
            params={'newTCCalendars': newTCCalendars},
            response_cls=TCCalendarResponse,
        )

    @classmethod
    def deleteTCCalendars(cls, tccalendarsToDelete: List[TCCalendar]) -> ServiceData:
        """
        This operation deletes the specified calendars and their calendar events based on the user's request to the
        application interface. Multiple calendars and its their events can be deleted   through this operation.
        Existing schedule, resource , and base calendars can be deleted through this operation.. 
        The information needed to delete a TCCalendar  object is specified in the TCCalendar structure. It returns 
        'ServiceData'  , which is  a common data structure used to return sets of Teamcenter data model objects from a
        service request. This structure holds lists of data model objects that were 'created', 'deleted', 'updated' or
        'plain' in the database with this service request. 'Plain' objects are simply objects that the service is
        returning where no changes have been made to the database object, i.e. e.g., 'GetHomeFolder' returns a list of
        objects that are contained in the user's home folder. Errors will be returned in the list of partial errors in
        the 'ServiceData'.   
        
        """
        return cls.execute_soa_method(
            method_name='deleteTCCalendars',
            library='CalendarManagement',
            service_date='2007_01',
            service_name='CalendarManagement',
            params={'tccalendarsToDelete': tccalendarsToDelete},
            response_cls=ServiceData,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class NameValueStringPair(TcBaseObj):
    """
    Structure containing a pair (string, string) of name and value respectively.
    
    :var name: The name of key in Pair.
    :var keyValue: The value of key in Pair.
    """
    name: str = ''
    keyValue: str = ''


@dataclass
class ResourceChartInfo(TcBaseObj):
    """
    This represents the UID of the resource which can be of type User, Group, ResourcePool or Discipline and the
    resource information for the given resource on a specific date.
    
    :var resource: The UID of User, Group, ResourcePool or Discipline to load resource information.
    :var resourceInfoPerDay: The list of resource information for each day in the start and end date range.
    """
    resource: str = ''
    resourceInfoPerDay: List[ResourceInfoPerDay] = ()


@dataclass
class ResourceChartResponse(TcBaseObj):
    """
    This contains the list of resource chart information per resource and ServiceData. The plain objects list of the
    ServiceData containing the list of assigned ScheduleTask objects and the resources.
    
    :var resourceChartInfoList: A list of Resource chart information per resource.
    :var serviceData: If the loadOptions contains &lsquo;loadAssignedTasks:true&rsquo;, then the assigned ScheduleTask
    objects will be added to the plain objects list of the ServiceData
    """
    resourceChartInfoList: List[ResourceChartInfo] = ()
    serviceData: ServiceData = None


@dataclass
class ResourceInfoPerDay(TcBaseObj):
    """
    The structure containing a date, resource information on the date and the list of ScheduleTask objects to the
    assigned to a resource.
    
    :var date: The date for which the resource information will be loaded.
    :var resourceInfo: The list of the resource information on the given date.  Examples of valid values are:
    availability :  120 (value is represented only in minutes)
    capacity :  480  (value is represented only in minutes)
    percentLoad : 75%
    :var assignedTasksInfo: A list of UIDs of assigned ScheduleTask objects on the given date and its associated
    information like resource load on the ScheduleTask.
    """
    date: datetime = None
    resourceInfo: List[NameValueStringPair] = ()
    assignedTasksInfo: List[TaskInfo] = ()


@dataclass
class TaskInfo(TcBaseObj):
    """
    This contains the information of a given ScheduleTask object.
    
    :var taskUid: The UID of the ScheduleTask object.
    :var info: The information for the assigned ScheduleTask object in key value pair  . Valid values are:
    percentLoad : 70%   (Valid values are represented as number greater than 0 which can include decimals. e.g. 5%,
    7.5%, 150%, etc.)
    """
    taskUid: str = ''
    info: List[NameValueStringPair] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from typing import Dict


@dataclass
class AnalyticsData(TcBaseObj):
    """
    This contains an instance of AnalyticsData required to hold software analytics data for logged in user.
    
    :var isDataCollectionEnabled: This is obtained from the value for preference TC_ProductExcellenceProgram to
    indicate the software analytics should be enabled or not.  If data collection is off then the server will not send
    any other information and an instance of AnalyticsData will be empty, this value is set to false.
    :var useInternalServer: This is used to indicate where the analytice data is logged. If true, send analytics data
    to analytics internal server; otherwise, send analytics data to analytics cloud based server (AWS).
    :var burstTimeInterval: The time interval at which software analytics data will be sent to the SA server. This
    value is a recommendation since all clients cannot implement a burst interval other than zero.
    :var analyticsExtraInfo: Map of key\value pairs ( string\string). These are optional fields sent by the server and
    will be sent to the analytics site as is. The intent is that these fields, if present, will be sent to the Software
    Analytics server retaining their name and value. Callers of this SOA should NOT use or rely on any values in this
    map. If isDataCollectionEnabled, this map will likely be empty. Any values passed in this map must have prior
    approval by Siemens Legal team.
    """
    isDataCollectionEnabled: bool = False
    useInternalServer: bool = False
    burstTimeInterval: int = 0
    analyticsExtraInfo: AnalyticsExtraInfo = None


@dataclass
class TCSessionAndAnalyticsInfo(TcBaseObj):
    """
    Data structure representing the current user's Teamcenter session information and software analytics related data.
    
    :var userSession: The user session object.
    :var extraInfoOut: Map of key/value pairs (string/string). Some/all/none of the following keys and values are
    returned, depending on what was passed in extraInfoIn:
    &bull;    TCServerVersion                  The version of the Teamcenter server.
    &bull;    hasProjects                "true" or "false" depending on whether the user has projects
    &bull;    AWC_StartupPreferences      This contains list of preferences to be retrieved at startup by the Active
    workspace client from the Teamcenter server.
    &bull;    AWC_Startuptypes        List of types to be loaded at start up by the Active workspace client from the
    Teamcenter server.
    &bull;    DefaultDateFormat        The default date format required.
    &bull;    AWServerVersion        The version of the Active workspace
    &bull;    typeCacheLMD        UTC formatted last modified date for Type cache Dataset.
    &bull;    WorkspaceId        Workspace Id for current Active workspace user session.
    &bull;    AWC_PostLoginStages    This preference represents list of post login stages in the sequence to be
    displayed in the Active worksapace after successful authentication.
    :var analyticsData: This contains an instance of AnalyticsData required to hold software analytics data for logged
    in user.
    :var serviceData: The service data.
    """
    userSession: BusinessObject = None
    extraInfoOut: SessionExtraInfo = None
    analyticsData: AnalyticsData = None
    serviceData: ServiceData = None


"""
Map of key/value pairs (string, string). Some, all, or none of the following keys and values are returned, depending on what was passed in extraInfoIn
&bull;    TCServerVersion                  The version of the Teamcenter server.
&bull;    hasProjects                "true" or "false" depending on whether the user has projects
&bull;    DefaultDateFormat        The default date format required.
&bull;    typeCacheLMD        UTC formatted last modified date for Type cache Dataset.
"""
SessionExtraInfo = Dict[str, str]


"""
Map of key\value pairs ( string\string). These are optional fields sent by the server and will be sent to the analytics site as is. The intent is that these fields, if present, will be sent to the Software Analytics server retaining their name and value. Callers of this SOA should NOT use or rely on any values in this map. If isDataCollectionEnabled, this map will likely be empty. Any values passed in this map must have prior approval by Siemens Legal team.
"""
AnalyticsExtraInfo = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.Internal.Integration._2007_06.IntegrationManagement import ConnectResponse
from tcsoa.base import TcService


class IntegrationManagementService(TcService):

    @classmethod
    def connect(cls, inputVal: int, action: str) -> ConnectResponse:
        """
        Perform the License related operation such as :
        ILM__init_module:   Initialize the license module if it has not already been initialized.
        ILM__leave_module:  De-allocate license of the given module. If the user had N free licenses for this module,
        he will have (N+1) left after this call
        ILM__check_module:  Check to see if the User has bought this module and return the number of licenses purchased
        ILM__enter_module:  Allocate one license of the given module. If the user has bought N licenses for this module,
        he will have (N-1) left after this call
        ILM__exit_module:   Leave the module.
        """
        return cls.execute_soa_method(
            method_name='connect',
            library='Internal-Integration',
            service_date='2007_06',
            service_name='IntegrationManagement',
            params={'inputVal': inputVal, 'action': action},
            response_cls=ConnectResponse,
        )



from __future__ import annotations

from tcsoa.gen.ConfigFilterCriteria._2011_06.EffectivityManagement import ConfigExpression, EffectivityTablesResponse
from tcsoa.gen.ConfigFilterCriteria._2017_05.EffectivityManagement import EffectivityConditionInfo, EffectivityConditionSource
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class EffectivityManagementService(TcService):

    @classmethod
    def setEffectivityConditionSubsets(cls, productName: str, productNamespace: str, subsetCriteria: ConfigExpression, actionCode: int, effectivityConditions: List[EffectivityConditionInfo]) -> ServiceData:
        """
        This operation sets the effectivity expression on the input business objects. The subsetCriteria parameter
        provides the effectivity subset for which the new input expression should be combined with the existing
        effectivity on the target object. Additionally the actionCode parameter controls how this should be done. If
        the subsetCriteria parameter is not provided then the operation assumes the complete effectivity expression of
        the input object should be considered.
        
        Use cases:
        Use case 1: User adding effectivity on the object having no existing effectivity. User does not provide
        subsetCriteria. The actionCode provided is 1 (Overwrite). 
        Input:
        Target Object existing effectivity:  None  
        Expression:  Unit = 3..9 &amp; Intent = Design
        Resultant Effectivity set on the object:   Unit = 3..9 &amp; Intent = Design
        
        Use case 2: User adding Effectivity on object with existing effectivity expression. User does not provide
        subsetCriteria. The actionCode provided is 1 (Overwrite).
        Input:
        Target Object existing effectivity:  Unit = 1..5 &amp; Intent = Design  
        Expression:  Unit = 3..9 &amp; Intent = Design
        Resultant Effectivity set on the object:   Unit = 3..9 &amp; Intent = Design
        
        Use case 3: User adding Effectivity on object with existing effectivity expression. User provides a
        subsetCriteria value. The actionCode provided is 1 (Overwrite).
        Input:
        Target Object existing effectivity:  Unit = 1...15 &amp; Intent = Design  
        Expression:  Unit = 3..9 &amp; Intent = Design
        Subset Criteria: Unit = 3..12 &amp; Intent = Design
        Resultant Effectivity set on the object:   Unit = 1..9  &amp; Intent = Design | Unit = 13..15 &amp; Intent =
        Design
        
        Use case 4: User adding Effectivity on object with existing effectivity expression. User provides a
        subsetCriteria value. The actionCode provided is 1 (Overwrite).
        Input:
        Target Object existing effectivity:  Unit = 1..15 &amp; Intent = Design | Unit = 5..10 &amp; Intent =
        Engineering
        Expression:  Unit = 3..9 &amp; Intent = Design
        Subset Criteria: Unit = 3..12 &amp; Intent = Design
        Resultant Effectivity set on the object:   Unit = 1..9  &amp; Intent = Design | Unit = 13..15 &amp; Intent =
        Design | Unit = 5..10 &amp; Intent = Engineering
        
        Use case 5: User adding Effectivity on object with existing effectivity expression. User provides a discrete
        subsetCriteria value. The actionCode provided is 1 (Overwrite).
        Input:
        Target Object existing effectivity:  Unit = 1..15 &amp; Intent = Design &amp; Unit = 5..10 &amp; Intent =
        Engineering
        Expression:  Unit = 3..6 &amp; Intent = Design
        Subset Criteria: Unit = 3..8 | Unit = 15..20 
        Resultant Effectivity set on the object:   Unit = 1..6 &amp; Intent = Design | Unit = 9..14 &amp; Intent =
        Design | Unit = 9..10 &amp; Intent = Engineering
        """
        return cls.execute_soa_method(
            method_name='setEffectivityConditionSubsets',
            library='ConfigFilterCriteria',
            service_date='2017_05',
            service_name='EffectivityManagement',
            params={'productName': productName, 'productNamespace': productNamespace, 'subsetCriteria': subsetCriteria, 'actionCode': actionCode, 'effectivityConditions': effectivityConditions},
            response_cls=ServiceData,
        )

    @classmethod
    def getEffectivitySubsetTables(cls, productName: str, productNamespace: str, subsetCriteria: ConfigExpression, effectivityConditions: List[EffectivityConditionSource]) -> EffectivityTablesResponse:
        """
        This operation retrieves the effectivity expression in the EffectivityTable form for the input business objects
        or ConfigExpression as input. If subset criteria are provided the operation only returns the subset that
        matches the subset criteria.
        
        Use cases:
        Use case 1.  User wants to retrieve the effectivity from the input object. The subsetCriteria is not provided.
               Input object effectivity: Unit = 1..5 &amp; Intent = Design
               Subset Criteria: None
        Operation Response:  
             Object Effectivity:  Unit =  1..5 &amp; Intent = Design
        
        Use case 2.  User wants to retrieve the effectivity from the input object. The subsetCriteria is provided with
        some value. The effectivity of the input object is a superset of the subsetCriteria.
                Input object effectivity: Unit = 1..15 &amp; Intent = Design
                Subset Criteria: Unit = 3..6 &amp; Intent = Design
        Operation Response:  
              Object Effectivity:  Unit = 3..6 &amp; Intent = Design
        
        Use case 3.  User wants to retrieve the effectivity from the input object. The subsetCriteria is provided with
        some value. The effectivity of the input object partially overlaps with the subsetCriteria.
                Input object effectivity: Unit = 1..15 &amp; Intent = Design
               Subset Criteria: Unit = 13..26 &amp; Intent = Design
        Operation Response:  
             Object Effectivity:  Unit = 13..15 &amp; Intent = Design
        
        Use case 4. User wants to retrieve the effectivity from the input object. The subsetCriteria is provided with
        some value. The effectivity of the input object does not overlap with the subsetCriteria.
                 Input object effectivity: Unit = 1..9 &amp; Intent = Design | Unit = 16..25 &amp; Intent = Engineering
                Subset Criteria: Unit = 10..15 
        Operation Response:  
           Object Effectivity:  None
        
        Use case 5.  User wants to retrieve the effectivity from the input object. The subsetCriteria is provided with
        some value. The effectivity of the input object partially overlaps with the subsetCriteria.
                Input object effectivity: Unit = 1..10 &amp; Intent = Design | Unit = 15..25 &amp; Intent = Engineering
                Subset Criteria: Unit = 3..18 &amp; Intent = Design | Unit = 10..20 &amp; Intent = Engineering
        Operation Response:  
           Object Effectivity:  Unit = 3..10 &amp; Intent = Design | Unit = 15..20 &amp; Intent = Engineering
        """
        return cls.execute_soa_method(
            method_name='getEffectivitySubsetTables',
            library='ConfigFilterCriteria',
            service_date='2017_05',
            service_name='EffectivityManagement',
            params={'productName': productName, 'productNamespace': productNamespace, 'subsetCriteria': subsetCriteria, 'effectivityConditions': effectivityConditions},
            response_cls=EffectivityTablesResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class SessionService(TcService):

    @classmethod
    def getProperties(cls, objects: List[BusinessObject], attributes: List[str]) -> ServiceData:
        """
        This is a wrapper to the 'DataManagement.getProperties' operation, and is provided only to give the client
        framework the ability to complete a partial response when the server side limits the size of the return payload.
        """
        return cls.execute_soa_method(
            method_name='getProperties',
            library='Internal-Core',
            service_date='2007_12',
            service_name='Session',
            params={'objects': objects, 'attributes': attributes},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class Action(TcBaseObj):
    """
    Contains the action to perform when the Awp0Tile is clicked in the Gateway.  If URL is populated then urlParams may
    optionally be populated depending on the Tile configuration.  If commandId is populated, then commandArgs may
    optionally be populated depending on the Awp0Tile configuration.
    
    If the Awp0TileTemplate object&rsquo;s awp0ActionType property is 0, 1 or 2 then the url property is populated.  If
    the Awp0TileTemplate object&rsquo;s awp0ActionType property is 3, then the commandId property is populated.
    
    :var actionType: Identifies the action type.   Possible values are 0 (default), 1 (external reference), 2 (static
    resource), 3 (command).
    :var url: The URL to navigate to in the client if commandId is undefined.
    :var commandId: The command ID to execute if URL is undefined.
    :var actionParams: Optional command parameters.  The names and values for this field come directly from the
    awp0Params property on the Awp0Tile object.  These params are an unconstrained list of name/value pairs.
    
    The syntax for awp0Params on the Awp0Tile object is in the format of
    <name>=<value>;<name2>=<value2>;<name3>=<value3>...  Where name/value pairs are delimited by semicolon.
    
    The awp0Params entry is parsed, and each name/value pair is added to a string map, where key is the parameter name,
    and the value is the parameter value.
    
    For example:  awp0Params may contain an entry such as "refresh=true;filter=POM_application_object.owning_user".  
    
    The resulting map would contain the following parsed pairs:
    
    "refresh":"true",
    "filter":"POM_application_object.owning_user"
    """
    actionType: int = 0
    url: str = ''
    commandId: str = ''
    actionParams: ActionParamMap = None


@dataclass
class GetCurrentUserGateway2Response(TcBaseObj):
    """
    GetCurrentUserGateway2Response viewmodel to be presented by the client.  The response consists of  TileGroup
    objects which contain Tile objects in the order they should be presented in the Gateway.   Each Tile object
    contains a reference to the Awp0GatewayTileRel so the Tile may be operated on.
    
    :var serviceData: Contains partial error information.
    :var tileGroups: List of TileGroup to be presented in the Gateway.
    """
    serviceData: ServiceData = None
    tileGroups: List[TileGroup] = ()


@dataclass
class PinObjectInput(TcBaseObj):
    """
    PinObjectInput contains the UID of the BusinessObject to pin, as well as optional data such as actionParams and
    templateId.   This information is used to create the Awp0Tile object in the server, and relate it to an 
    Awp0TileTemplate.
    
    :var uid: The UID of the object to pin.
    :var actionParams: Optional parameters to set on the Awp0Tile object.
    
     The syntax for actionParams is in the format of <name>=<value>;<name2>=<value2>;<name3>=<value3>...  Where
    name/value pairs are delimited by semicolon.
    :var templateId: Optional templateId to set on the Awp0Tile object. The value shall point to a valid template ID
    which is defined by property, awp0TemplateId  on an Awp0TileTemplate object.   If the template ID isn&rsquo;t
    valid, then no template is set on the Awp0Tile object.  If the value is an empty string, then the
    Awp0PinnedObjectTemplate ID is used by default.
    """
    uid: str = ''
    actionParams: str = ''
    templateId: str = ''


@dataclass
class SaveViewModelEditAndSubmitResponse2(TcBaseObj):
    """
    Structure containing saveViewModelEditAndSubmit2 operation response.
    
    :var workflowProcess: Workflow template object for each input object that will be created after it is submitted to
    workflow successfully.
    :var serviceData: Partial errors are returned in the Service Data.
    :var viewModelObjectsJsonString: viewModelObjectsJsonString A JSON string that contains the ViewModelObject
    structures which provide information about the editable status of the properties passed in the input. This
    information can be used by client to show which properties as editable. The ViewModelObjectJsonString is also used
    in loadViewModelForEditing2 and getDeclarativeStyleSheet2 operation which displays the ViewModelObject.
    """
    workflowProcess: List[BusinessObject] = ()
    serviceData: ServiceData = None
    viewModelObjectsJsonString: List[str] = ()


@dataclass
class Tile(TcBaseObj):
    """
    Contains all the information relevant to presenting and operating on the Tile object in the UI.
    
    :var displayName: The display name of the Tile object.
    :var themeIndex: The integer that indicates the theme for color of the Tile. Valid values are presented via a list.
    0: light, 1 medium, 2: highlight, 3: alternate.
    :var icons: Defines the primary and secondary icons to be presented in the Gateway.
    :var action: The action to execute when the Tile is clicked.
    :var tileSize: Size of the Tile. Valid values are 0, 1, or 2 (small, wide, or large).Null value not allowed.
    :var styleOverride: This property indicates any CCS classes to override the default styling of the Tile.
    :var orderNumber: The order number for which this Tile should be presented in the Gateway.
    :var content: Defines the live content to be displayed on a Tile.  Each entry is a line on the Tile.
    :var isProtected: Indicates whether a Tile may be removed from a TileGroup.
    :var validSizes: Sizes of the Tile that this template supports. Can be used to limit available sizes for a tile
    type - if blank, all sizes are allowed. Valid values are presented via a list. 0: small, 1: wide, 2: large.
    :var relUID: The UID of the Awp0GatewayTileRel object  which describes the Tile/TileGroup information.
    """
    displayName: str = ''
    themeIndex: int = 0
    icons: TileIcons = None
    action: Action = None
    tileSize: int = 0
    styleOverride: str = ''
    orderNumber: int = 0
    content: List[ContentEntry] = ()
    isProtected: bool = False
    validSizes: List[int] = ()
    relUID: str = ''


@dataclass
class TileGroup(TcBaseObj):
    """
    Contains the groupName to be presented in the UI, as well as the vector of Tile objects to be presented in the UI. 
    The Tile objects are ordered based on awp0OrderNo property on the Awp0GatewayTileRel object.
    
    :var groupName: The name of the TileGroup.
    :var tiles: The Tile objects to be presented in the TileGroup.
    """
    groupName: str = ''
    tiles: List[Tile] = ()


@dataclass
class TileIcons(TcBaseObj):
    """
    This class represents the icons to be presented for a Tile object.
    
    :var primaryIcon: The primary icon to display on a Tile. Valid values are a file ticket, a static icon name, or
    "__TYPEICON__".  The value of "__TYPEICON__" indicates that the type hierarchy must be walked into order to
    determine the icon to present in the UI.  The 1st icon that it finds in the type hierarchy is the one to use.
    :var secondaryIcon: The secondary icon to display on a Tile. Valid values are a file ticket, a static icon name, or
    "__TYPEICON__". The value of "__TYPEICON__" indicates that the type hierarchy must be walked into order to
    determine the icon to present in the UI.  The 1st icon that it finds in the type hierarchy is the one to use.
    :var typeHierarchy: The type hierarchy for the Tile, if the Tile represents a pinned object. It is empty otherwise.
    If the value of primaryIcon or secondaryIcon is "__TYPEICON__" then this field is used by the client to determine
    which icon to present on the Tile.
    """
    primaryIcon: str = ''
    secondaryIcon: str = ''
    typeHierarchy: List[str] = ()


@dataclass
class UnpinObjectsResponse(TcBaseObj):
    """
    UnpinObjectsResponse contains the gateway viewmodel to be presented by the client.  The response consists of 
    TileGroup objects which contain Tile objects in the order they should be presented in the Gateway. Each Tile object
    contains a reference to the Awp0GatewayTileRel so the Tile may be operated on.
    
    :var tileGroups: List of TileGroup to be presented in the Gateway.
    :var serviceData: Contains partial error information.
    """
    tileGroups: List[TileGroup] = ()
    serviceData: ServiceData = None


@dataclass
class UpdateTilesResponse(TcBaseObj):
    """
    UpdateTilesResponse contains the gateway viewmodel to be presented by the client. The response consists of
    TileGroup objects which contain Tile objects in the order they should be presented in the Gateway. Each Tile object
    contains a reference to the Awp0GatewayTileRel so the Tile may be operated on.
    
    :var tileGroups: List of TileGroup objects to be presented in the Gateway.
    :var serviceData: Contains partial error information.
    """
    tileGroups: List[TileGroup] = ()
    serviceData: ServiceData = None


@dataclass
class ContentEntry(TcBaseObj):
    """
    Contains live Tile information to be presented on Gateway Tile objects, how many items are in the user&rsquo;s
    inbox, for example.
    
    :var contentName: The name to be presented
    :var contentValue: The value to be presented
    """
    contentName: str = ''
    contentValue: str = ''


"""
Contains optional command information to execute after the url or command ID is executed.
"""
ActionParamMap = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Item
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class ImportExportService(TcService):

    @classmethod
    def registerItemId(cls, itemInfo: List[Item]) -> ServiceData:
        """
        The 'registerItemId' operation is used to register the list of items at the central item registry.  The central
        registry holds all item ids from the Multi-Site federation. The item ID could be manually registered using Rich
        client, Command line utility data_share OR using this operation.
        
        Usage of this operation is influenced by the following preferences.
         
        The preference ITEM_id_registry activates/deactivates the central item registry.  If the preference value is
        set to TRUE, during item creation, the system checks the item registry for duplicates. Creating the item fails
        if a duplicate is found. Setting ITEM_id_Registry to FALSE would make Item registry disabled. Thus, while
        creating a new item, the item registry is not checked for duplicates.
        
        The preference ITEM_id_registry_site identifies the site name at which the central item registry server is
        running. This must be set, if the preference ITEM_id_registry is set to TRUE and will be ignored otherwise.
         
        The preference ITEM_id_allow_if_registry_down determines if items can be created, if the registry server is not
        available. If the registry is not active (ITEM_id_registry = FALSE), this preference is ignored.
        
        The preference ITEM_id_always_register_on_creation determines if item IDs are automatically registered when
        items are created or the item ID is changed.
        
        Use cases:
        - Admin user(s) set the ITEM_id_always_register_on_creation to TRUE on all sites within a Multi-Site
        federation. On item creation the items created automatically registers with the central item registry.
        - Admin user(s) can register existing item(s) with the central item registry, using Register Item Id command in
        the Rich Client(RAC) or command line utility data_share f=register
        
        """
        return cls.execute_soa_method(
            method_name='registerItemId',
            library='Multisite',
            service_date='2011_06',
            service_name='ImportExport',
            params={'itemInfo': itemInfo},
            response_cls=ServiceData,
        )

    @classmethod
    def unregisterItemId(cls, itemInfo: List[Item]) -> ServiceData:
        """
        The central registry holds all item ids from the multisite federation. To unregister the item id(s), this
        operation is used, this unregisters supplied items from the central item registry. The item ID could be
        manually unregistered using Rich client, Command line utility data_share OR using this operation.
        
        The preference ITEM_id_registry activates/deactivates the central item registry.  If the preference value set
        to TRUE, during item creation, the system checks the item registry for duplicates. Creating the item fails if a
        duplicate is found. Setting ITEM_id_Registry to FALSE would make item registry disabled. So while creating a
        new item, the item registry is not checked for duplicates. 
        
        The preference ITEM_id_registry_site identifies the site name at which the central item registry server is
        running. This must be set, if the preference ITEM_id_registry is set to TRUE and will be ignored otherwise.
        
        The preference ITEM_id_unregister_on_delete determines if item Ids are automatically unregistered when items
        are deleted or the item ID is changed.
        
        
        
        Use cases:
        - Admin user(s) can unregister existing item(s) from the central item registry, using Unregister Item Id
        command in the Rich Client (RAC) or command line utility data_share f=unregister.
        
        """
        return cls.execute_soa_method(
            method_name='unregisterItemId',
            library='Multisite',
            service_date='2011_06',
            service_name='ImportExport',
            params={'itemInfo': itemInfo},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import User, Group, Role
from tcsoa.gen.Core._2009_04.ProjectLevelSecurity import LoadProjectDataForUserResponse
from tcsoa.base import TcService


class ProjectLevelSecurityService(TcService):

    @classmethod
    def loadProjectDataForUser(cls, user: User, group: Group, role: Role) -> LoadProjectDataForUserResponse:
        """
        This operation returns list of projects for a given user, group and role combination. If no group and role is
        specified it obtains all the projects for the specified user. If any of the arguments passed are invalid an
        error is returned by the operation added as a partial error.
        """
        return cls.execute_soa_method(
            method_name='loadProjectDataForUser',
            library='Core',
            service_date='2009_04',
            service_name='ProjectLevelSecurity',
            params={'user': user, 'group': group, 'role': role},
            response_cls=LoadProjectDataForUserResponse,
        )


class SessionService(TcService):

    @classmethod
    def startOperation(cls) -> str:
        """
        Start an operation bracket.  An operation bracket is a period of execution in which any object will need to be
        refreshed in the server from the database only once.  This allows the client to avoid unnecessary database
        operations that the server might perform redundantly if underlying code accesses the same object multiple
        times.  The client will use the return value to call the 'stopOperation' operation to indicate the end of the
        bracket.  Brackets may be nested or overlapped.  A bracket should start and end within the scope of a single
        client function and should not span a user interaction.  By default, each service operation starts and stops
        its own operation bracket.
        """
        return cls.execute_soa_method(
            method_name='startOperation',
            library='Core',
            service_date='2009_04',
            service_name='Session',
            params={},
            response_cls=str,
        )

    @classmethod
    def stopOperation(cls, opId: str) -> bool:
        """
        Stop an operation bracket, in which objects need to be refreshed only once.  See 'startOperation'.
        """
        return cls.execute_soa_method(
            method_name='stopOperation',
            library='Core',
            service_date='2009_04',
            service_name='Session',
            params={'opId': opId},
            response_cls=bool,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import WorkspaceObject, ImanType, IdContext, Item, ItemRevision
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class IDContextOut(TcBaseObj):
    """
    The IDContextOut is list of  IdContext objects. It also contains the input object of type Item or ItemRevision.
    
    :var inputObj: Input object of type WorkspaceObject or null for which the list of IdContext objects are fetched.
    :var idContexts: A list of all IdContext objects found in the Teamcenter database.
    """
    inputObj: WorkspaceObject = None
    idContexts: List[IdContext] = ()


@dataclass
class IDContextOutput(TcBaseObj):
    """
    The IDContextOutput is list of IdContextOut objects. It also contains the instance of ServiceData for the operation.
    
    :var idContextOuts: A list of IdContextOut structures which holds the list of IdContext objects and input object of
    type WorkspaceObject or null.
    :var serviceData: Any partial errors encountered during this operation are returned in the list of partial errors
    of the ServiceData.
    """
    idContextOuts: List[IDContextOut] = ()
    serviceData: ServiceData = None


@dataclass
class IDDispRuleCreateIn(TcBaseObj):
    """
    The IDDispRuleCreateIn structure is used to hold the name of the ID Display Rule, list of ID Context objects and a
    flag whether to set the ID Display Rule being created as the default ID Display Rule.
    
    :var ruleName: Name of the ID Display Rule being created.
    :var idContexts: A list of input IdContext objects to be set with the IdDispRule object being created.
    :var useDefault: If true, the created IdDispRule object is set as the default ID Display Rule for the user;
    otherwise, the existing default ID Display Rule will remain the default.
    :var setCurrent: If true, the created IdDispRule object is set as the current ID Display Rule for the user;
    otherwise, the existing current ID Display Rule will remain the current.
    """
    ruleName: str = ''
    idContexts: List[IdContext] = ()
    useDefault: bool = False
    setCurrent: bool = False


@dataclass
class IdentifierTypesIn(TcBaseObj):
    """
    The IdentifierTypesIn structure is used to hold the input Item or ItemRevision object and the input IdContext
    object.
    
    :var inputItemOrRev: Input Item or ItemRevision object for which the Identifier type is being fetched.
    :var idContext: Input IdContext object used to fetch the Identifier type for the input Item or ItemRevision from
    the defined ID Display Rules.
    """
    inputItemOrRev: WorkspaceObject = None
    idContext: IdContext = None


@dataclass
class IdentifierTypesOut(TcBaseObj):
    """
    The IdentifierTypesOut is list of IdentifiersOut objects. It also contains the instance of ServiceData for the
    operation.
    
    :var identifiersOutput: A list of IdentifiersOut structures which holds list of Identifier types, object of type
    Item and list of ItemRevision objects, applicable for defining the Alternate IDs.
    :var serviceData: Any partial errors encountered during this operation are returned in the list of partial errors
    of the ServiceData.
    """
    identifiersOutput: List[IdentifiersOut] = ()
    serviceData: ServiceData = None


@dataclass
class IdentifiersOut(TcBaseObj):
    """
    The IdentifiersOut structure is used to hold the list of Identifier types, applicable object of type Item and list
    of ItemRevision objects, applicable for defining the Alternate IDs.
    
    :var inputItemOrRev: Input Item or ItemRevision object for which the Identifier type is being fetched.
    :var identifierTypes: A list of valid Identifier types which can be used to define the Alternate and Alias IDs of
    the input object.
    :var item: Item object of the given input ItemRevision for which Alternate IDs can be defined.
    :var revisions: A list containing all revisions of the input Item for which Alternate IDs can be defined.
    """
    inputItemOrRev: WorkspaceObject = None
    identifierTypes: List[ImanType] = ()
    item: Item = None
    revisions: List[ItemRevision] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ImportExportOptionsInfo(TcBaseObj):
    """
    The ImportExportOptionsInfo structure holds the name-value pairs of import/export option(s). Value can be single or
    multiple valued. The import/export options influence the business object import/export and has default value.
    
    :var optionName: The name of the option. Valid values are as per below table.
    
    Supported ImportExportOptionsInfo options:  
    
                  Option Name                  Description                                                             
    Default value
    opt_exp_prot_obj        Set this option to False so process will stop when exporting import 
                            or export protected workspace objects.                                                  
    False
    opt_do_struct           Set this option to TRUE to Transfer site ownership of the Top-Level
                            item only and export all components with no site ownership transfer. 
                            Applicable only if transfer_ownership is true.                                          
    False 
    opt_trans_prot_comp     Set this option to TRUE to exclude all components that have no 
                            TRANSFER_OUT/TRANSFER_IN privilege at owning site 
                            for site ownership transfer.
                            Applicable only if transfer_ownership is true.                                          
    False 
    opt_exp_prot_comp       Set this option to TRUE to exclude all components that have no export
                            and/or import privileges granted at the owning site.                                    
    False
            
    opt_entire_bom             Set this option to TRUE to export all components if the item selected is an assembly.   
      False 
    opt_all_ds_files         Set this option to TRUE to export all dataset files.                                    
    True 
    opt_all_ds_vers         Set this option to TRUE to export all dataset versions                                     
    True 
    opt_folder_contents     Set this option to TRUE to export objects in folder.                                     
    False 
    opt_de_rlz_item         Set this option to TRUE to send BOM structure and CPD structure together                 
    False 
    opt_workset_rlz_de         Set this option to TRUE to send Design Element with Workset and subset                  
       False 
    opt_cpd_p2s_skip         Set this option to True to exclude all Reference Geometry/Connected Element 
                            relations from Cpd0DesignElement.                                                        
    True 
    transfer_ownership      Set this option to True to transfer site ownership of selected objects.                    
    False 
    owning_user                Set the owning user for objects at remote site                                          
      None
    owning_group            Set the owning group for objects at the remote site                                        
    None
    include_modified_only     Include only objects that have been modified since the last transfer are exported.
                            Mutually exclusive with the  transfer_ownership option.                                 
    False
    include_relations       List of relations which will be included in export. 
                            The relation must be passed by Teamcenter relation name 
                            and be supported for include processing by the option set.                                
    None
    exclude_relations        List of relations which will be excluded in export. 
                            The relation must be passed by Teamcenter relation name 
                            and be supported for exclude processing by the option set.                                
    None
            
    
    Item Options, only one may be passed. 
    Option Name                              Description                                                               
                                                                         Value(s)
    opt_rev_select                           Process all revisions.                                                    
                                                                         allItemRevisions   
    opt_rev_select                              Process only the latest revision regardless of release status.         
                                                               latestRevisionOnly
    opt_rev_select                           Process only the latest revision regardless of release status.            
                                                            latestWorkingRevisionOnly
    opt_rev_select                           Process only the latest working revision. If there is none, then process
    the latest released                                 latestWorkingAnyOnly
    opt_rev_select                           Process only the latest released revision.                                
                                                                    latestReleasedRevisionOnly
    opt_rev_select                           Process only the selected revision. Applicable only if an ItemRevision is
    the target.                                         selectedRevisionsOnly
    opt_rev_select                           Process only revisions with a specific release status. Supported release
    statuses are defined in the option set.       First value=specificStatusOnly Second value=Release Status Name 
    :var optionValue: A list of values for the optionName.
    """
    optionName: str = ''
    optionValue: List[str] = ()


@dataclass
class RemoteExportFailurePerOperation(TcBaseObj):
    """
    Returns failures from all sites during export of objects.
    
    :var sites: A list of sites by name for this operation.
    :var failuresPerOperation: A list of failure information from sites.
    """
    sites: List[str] = ()
    failuresPerOperation: List[RemoteExportFailurePerSite] = ()


@dataclass
class RemoteExportFailurePerSite(TcBaseObj):
    """
    This structure returns failures encountered during export to a site.
    
    The following errors may be returned in the RemoteExportFailurePerSite structure.
    
    96001 - The bulk "Iman Export Record" (IXR) creation or update has failed during the TC XML based multisite
    operation. 
    96002 - The "Export Commit" process has failed during the TC XML based multisite operation. 
    96003 - The processing callback function is not set for the TC XML based multisite operation.
    96004 - The "Export Record" could not be created during the TC XML based multisite operation. 
    96005 - The "Export Record" could not be created during the TC XML based multisite operation due to an invalid
    value for the attribute 
    96023 - An error has been encountered during the access of the volume during the TC XML based operation. 
    96024 - An invalid input data was found while updating the fast synchronization tables during the TC XML based
    operation. Please refer to the log files for details.
    96025 - The save of "Iman Export Record" (IXR) has encountered partial errors during the TC XML based multisite
    operation. Please refer to the log files for details.
    96026 - The conversion of the input parameters of the utility to its representative Option Set has failed.
    
    
    :var failureCodes: A list of failure codes for this operation.
    :var failureObjects: A list of objects that have failed to export.
    :var failureStrings: A list of error strings for this operation.
    """
    failureCodes: List[int] = ()
    failureObjects: List[BusinessObject] = ()
    failureStrings: List[str] = ()


@dataclass
class RemoteExportInfo(TcBaseObj):
    """
    The RemoteExportInfo structure takes an option set name and export options as input. The option set and options
    will affect the final state of the exported objects at the target sites. There is system default option set and
    additional option sets may be created by the administrator.  When the option set and options are not specified in
    the RemoteExportInfo structure, the system default option set and default options will be used.
    
    :var objects: A list of target objects.
    :var targetSites: A list of target sites by site name.
    :var optionSetName: Name of option set used for export.
    :var reason: The reason for export. It may be empty.
    :var exportOptions: A list of export options.
    """
    objects: List[BusinessObject] = ()
    targetSites: List[str] = ()
    optionSetName: str = ''
    reason: str = ''
    exportOptions: List[ImportExportOptionsInfo] = ()


@dataclass
class RemoteExportResponse(TcBaseObj):
    """
    The RemoteExportResponse returns detailed partial failure information in RemoteExportFailurePerOperation structures
    along with a ServiceData.
    
    This operation can return errors from different sites and each target object may have different errors. The
    RemoteExportFailurePerOperation structure will be used to report failures instead of the ServiceData to allow for
    the detailed reporting of site specific errors.  The RemoteExportFailurePerOperation structure will report what
    which site the errors are returned from. The list RemoteExportFailurePerSite structures will return errors for a
    specific target object(s). 
    
    
    :var failureInfo: The partial failure information specfic to site.
    :var serviceData: The Service Data return. The objects sucessfully exported are added to the Plain objects list.  
    """
    failureInfo: List[RemoteExportFailurePerOperation] = ()
    serviceData: ServiceData = None


@dataclass
class RemoteImportFailurePerOperation(TcBaseObj):
    """
    This structure returns all the failures during import for a set of objects
    
    :var sites: A list of sites by name for this set of failure data.
    :var failuresPerOperation: A list of failure information from sites.
    """
    sites: List[str] = ()
    failuresPerOperation: List[RemoteImportFailurePerSite] = ()


@dataclass
class RemoteImportFailurePerSite(TcBaseObj):
    """
    following errors may be returned in the RemoteImportFailurePerSite structure.
    
    96001 - The bulk "Iman Export Record" (IXR) creation or update has failed during the TC XML based multisite
    operation. 
    96002 - The "Export Commit" process has failed during the TC XML based multisite operation. 
    96003 - The processing callback function is not set for the TC XML based multisite operation.
    96004 - The "Export Record" could not be created during the TC XML based multisite operation. 
    96005 - The "Export Record" could not be created during the TC XML based multisite operation due to an invalid
    value for the attribute 
    96023 - An error has been encountered during the access of the volume during the TC XML based operation. 
    96024 - An invalid input data  was found while updating the fast synchronization tables during the TC XML based
    operation. Please refer to the log files for details.
    96025 - The save of "Iman Export Record" (IXR) has encountered partial errors during the TC XML based multisite
    operation. Please refer to the log files for details.
    96026 - The conversion of the input parameters of the utility to its representative Option Set has failed.
    
    
    :var failureCodes: A list of failure codes for this operation.
    :var failureObjects: A list of objects that have failed to import
    :var failureStrings: A list of error strings for this operation.
    """
    failureCodes: List[int] = ()
    failureObjects: List[BusinessObject] = ()
    failureStrings: List[str] = ()


@dataclass
class RemoteImportInfo(TcBaseObj):
    """
    The RemoteImportInfo structure takes an option set name and export options as input. The option set and options
    will affect the final state of the exported objects at the target sites. There is system default option set and
    additional option sets may be created by the administrator. When the option set and options are not specified in
    the RemoteExportInfo structure, the system default option set and default options will be used. Please find the
    default option values in the list of supported options below. 
    The targeted objects can be of two types.
    1.    A replica PomObject which has been previously imported into the local site
    2.    A PublishedObject which is proxy for remote object. This object must minimally contain UID and the owning
    site information. 
    
    
    :var objects: A list of target objects.
    :var optionSetName: Name of option set used for import.
    :var reason: The reason for this import. It may be empty.
    :var importOptions: A list of import options.
    """
    objects: List[BusinessObject] = ()
    optionSetName: str = ''
    reason: str = ''
    importOptions: List[ImportExportOptionsInfo] = ()


@dataclass
class RemoteImportResponse(TcBaseObj):
    """
    The RemoteImportResponse returns detailed partial failure information in RemoteImportFailurePerOperation structures
    along with a ServiceData. 
    This operation can return errors from different sites and each target object may have different errors per site. 
    The RemoteImportFailurePerOperation structure will be used to report failures instead of the ServiceData to allow
    for the detailed reporting of site specific errors.  The RemoteImportFailurePerOperation structure will report what
    which site the errors are returned from. The list RemoteImportFailurePerSite structures will return errors for a
    specific target object(s). 
    
    
    :var failureInfo: The partial failure information specfic to site.
    :var serviceData: The Service Data return. Objects sucessfully imported will be added to the Updated object list.
    """
    failureInfo: List[RemoteImportFailurePerOperation] = ()
    serviceData: ServiceData = None


@dataclass
class RemoteImportUIDInfo(TcBaseObj):
    """
    The RemoteImportUIDInfo structure takes an option set name and export options as input. The option set and options
    will affect the final state of the exported objects at the target sites. There is system default option set and
    additional option sets may be created by the administrator. When the option set and options are not specified in
    the RemoteExportInfo structure, the system default option set and default options will be used. 
    
    :var objectUIDs: A list of target objects in UID form.
    :var owningSiteIds: A list of owning sites by site ids for each object in objectUIDs
    :var optionSetName: Name of option set used for import.
    :var reason: The reason for this import. It may be empty.
    :var importOptions: A list of import options.
    """
    objectUIDs: List[str] = ()
    owningSiteIds: List[int] = ()
    optionSetName: str = ''
    reason: str = ''
    importOptions: List[ImportExportOptionsInfo] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class FileTicket(TcBaseObj):
    """
    To represent a file ticket and its original file name.
    
    :var ticket: The FMS file Ticket.
    :var fileName: The original file name.
    """
    ticket: str = ''
    fileName: str = ''


@dataclass
class GeneralInfo(TcBaseObj):
    """
    Holds all additional flags that are needed for the connect like quantity (number of occurrences),  copy predecessor
    flag, copy occurrence type from the source flag, propagate XForm flag.
    
    :var stringProps: Map containing string property values
    :var stringArrayProps: Map containing string array property values
    :var dateProps: Map containing DateTime property values
    :var dateArrayProps: Map containing DateTime array property values
    :var tagProps: Map containing string property values
    :var tagArrayProps: Map containing string array property values
    :var doubleProps: Map containing string property values
    :var doubleArrayProps: Map containing string array property values
    :var floatProps: Map containing string property values
    :var floatArrayProps: Map containing string array property values
    :var intProps: Map containing string property values
    :var intArrayProps: Map containing string array property values
    :var boolProps: Map containing string property values
    :var boolArrayProps: Map containing string array property values
    """
    stringProps: StringMap1 = None
    stringArrayProps: StringVectorMap1 = None
    dateProps: DateMap1 = None
    dateArrayProps: DateVectorMap1 = None
    tagProps: TagMap1 = None
    tagArrayProps: TagVectorMap1 = None
    doubleProps: DoubleMap1 = None
    doubleArrayProps: DoubleVectorMap1 = None
    floatProps: FloatMap1 = None
    floatArrayProps: FloatVectorMap1 = None
    intProps: IntMap1 = None
    intArrayProps: IntVectorMap1 = None
    boolProps: BoolMap1 = None
    boolArrayProps: BoolVectorMap1 = None


@dataclass
class GetAssociatedContextsInputData(TcBaseObj):
    """
    GetAssociatedContextsInputData
    
    :var associateToContext: associateToContext
    :var relationName: relationName
    """
    associateToContext: BusinessObject = None
    relationName: List[str] = ()


@dataclass
class AssociateAndAllocateInput(TcBaseObj):
    """
    Input for the automatic allocation commands such as automaticAssociateAndAllocate, associateAndAllocatePreview and
    associateAndAllocateByPreview command.
    
    :var sourceProductBOP: Source Product BOP for which allocate command was called
    :var targetPlantBOPLines: Target Plant BOP's lines to which the allocation is to be done.
    :var referenceProductBOP: Reference Product BOP according to which allocation is to be done.
    """
    sourceProductBOP: BusinessObject = None
    targetPlantBOPLines: List[BusinessObject] = ()
    referenceProductBOP: BusinessObject = None


@dataclass
class AssociateAndAllocateResponse(TcBaseObj):
    """
    Response for the automatic allocation commands such as automaticAssociateAndAllocate, and
    associateAndAllocateByPreview.
    
    :var serviceData: Service data will hold partial errors, warnings  and errors, if any.
    :var logFileTicket: File Ticket Containing the UID and file name for the Log File generated for this command.
    """
    serviceData: ServiceData = None
    logFileTicket: FileTicket = None


@dataclass
class AssociateOutput(TcBaseObj):
    """
    Associated BOMLines to the input context
    
    :var context: Input context
    :var associatedContextsInfo: vector of pairs of contexts and relation name that are associated with the input
    context
    """
    context: BusinessObject = None
    associatedContextsInfo: List[AssociatedContextInfo] = ()


@dataclass
class AssociatedContextInfo(TcBaseObj):
    """
    AssociatedContextInfo
    
    :var context: context  The added (source) context that should be associated (linked) associate to target context
    (associateToContext parameter).
    :var relationName: relationName  The relation to use to connect. If the string is empty then the relation defined
    as default will be used. It should be possible to define the default association relation for each pair of types.
    """
    context: BusinessObject = None
    relationName: str = ''


@dataclass
class AssociationResponse(TcBaseObj):
    """
    Contains a list of items tags representing the associated BOMLines with the given context.
    
    :var output: A vector of BOMLines associated to the input context
    :var serviceData: Partial errors mapped to the client id
    """
    output: List[AssociateOutput] = ()
    serviceData: ServiceData = None


@dataclass
class AutomaticAllocatePreviewResponse(TcBaseObj):
    """
    Response for the automatic allocation commands automaticAllocatePreview.
    
    :var serviceData: Service data will hold partial errors, warnings  and errors, if any.
    :var previewFileTicket: File Ticket Containing the UID and file name for the CSV File generated for preview during
    this command.
    :var allocationMap: Map of allocations from the source structure to the target structure.
    """
    serviceData: ServiceData = None
    previewFileTicket: FileTicket = None
    allocationMap: AllocationMap = None


@dataclass
class SourceInfo(TcBaseObj):
    """
    contains all needed information in order to connect source BOMLines to the target
    
    :var sourceObjects: BOMLines to connect the target
    :var relationType: occurrence type for new assigned line. If its empty  server will use the default occurrence type
    :var relationName: The relation for the connect. If the string is empty then the default relation will be defined
    and used by the server
    :var additionalInfo: Holds all additional flags that are needed for connect like quantity (number of occurrences), 
    copy predecessor flag, copy occurrence type from the source flag, propagate XForm flag. Can be easily extended
    without changing the signature of the SOA
    """
    sourceObjects: List[BusinessObject] = ()
    relationType: str = ''
    relationName: str = ''
    additionalInfo: GeneralInfo = None


@dataclass
class ConnectObjectResponse(TcBaseObj):
    """
    ConnectObjectResponse
    
    :var newObjects: new BOMLines, created as a result of connection operation under the new target BOMLines
    :var serviceData: serviceData
    """
    newObjects: List[BusinessObject] = ()
    serviceData: ServiceData = None


@dataclass
class ConnectObjectsInputData(TcBaseObj):
    """
    A list of ConnectInput BOMLines for the nodes to be connect
    
    :var targetObjects: BOMLines to connect with. Can be a vector as a result of the multiple selection of the targets
    :var sourceInfo: all needed information in order to connect source BOMLines to the target
    """
    targetObjects: List[BusinessObject] = ()
    sourceInfo: SourceInfo = None


@dataclass
class AddAssociationInput(TcBaseObj):
    """
    BOMLines for the contexts to be associated
    
    :var associateToContext: target (primary) context to associate (link) to.
    :var addedContexts: vector of contexts info that are going to be associated(added) to the target.
    """
    associateToContext: BusinessObject = None
    addedContexts: List[AssociatedContextInfo] = ()


@dataclass
class DisconnectFromOriginInputData(TcBaseObj):
    """
    Input structure for DisconnectFromOrigin Command
    
    :var lineToDisconnect: Plant BOP or Product BOP line that needs to be disconnected from their origin.
    :var isRecursive: Flag specifying whether the sub-processes and sub-operations are to be disconnected recursively.
    """
    lineToDisconnect: BusinessObject = None
    isRecursive: bool = False


"""
DateMap
"""
DateMap1 = Dict[str, datetime]


"""
DateVectorMap
"""
DateVectorMap1 = Dict[str, List[datetime]]


"""
DoubleMap
"""
DoubleMap1 = Dict[str, float]


"""
DoubleVectorMap
"""
DoubleVectorMap1 = Dict[str, List[float]]


"""
FloatMap
"""
FloatMap1 = Dict[str, float]


"""
FloatVectorMap
"""
FloatVectorMap1 = Dict[str, List[float]]


"""
IntMap
"""
IntMap1 = Dict[str, int]


"""
IntVectorMap
"""
IntVectorMap1 = Dict[str, List[int]]


"""
StringMap
"""
StringMap1 = Dict[str, str]


"""
BoolMap
"""
BoolMap1 = Dict[str, bool]


"""
StringVectorMap
"""
StringVectorMap1 = Dict[str, List[str]]


"""
TagMap
"""
TagMap1 = Dict[str, BusinessObject]


"""
TagVectorMap
"""
TagVectorMap1 = Dict[str, List[BusinessObject]]


"""
BoolVectorMap
"""
BoolVectorMap1 = Dict[str, List[bool]]


"""
Map of allocations from the source structure to the target structure. Relevant to the commands such as associateAndAllocateByPreview
"""
AllocationMap = Dict[BusinessObject, List[BusinessObject]]



from __future__ import annotations

from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetPersonPropertiesResponse(TcBaseObj):
    """
    This structure holds the list of PropertyValuesData for each of the requested person objects.
    
    :var serviceData: The object which holds the partial errors while getting the property values.
    :var personData: The list of PropertyValuesData objects which holds the values of the given properties for all the
    given person business objects.
    """
    serviceData: ServiceData = None
    personData: List[PropertyValuesData] = ()


@dataclass
class PropertyValuesData(TcBaseObj):
    """
    This structure holds the retrieved values of the given properties for a Person object.
    
    :var propValues: The list of property values of a Person object.
    """
    propValues: List[str] = ()


"""
Name and Value pair for properties.
"""
NameValueMap = Dict[str, str]



from __future__ import annotations

from typing import Dict, List
from tcsoa.gen.Server import PartialErrors
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class Feature(TcBaseObj):
    """
    Information for a single client cache feature.
    
    :var name: Name of the feature,  ClientMetaModel or TextData.
    :var cacheTickets: A map of Dataset names and FMS tickets (string/string) for each Dataset that makes up the client
    cache feature.
    """
    name: str = ''
    cacheTickets: CacheTickets = None


@dataclass
class LoginResponse(TcBaseObj):
    """
    Basic information about the server and  partial errors are returned when the authentication is successful but
    requested 'role' or 'locale' is not supported.
    
    :var serverInfo: Name/Value pairs (string/string) of data related to the server. The following keys are valid:
    - Version         The version of the Teamcenter server.
    - HostName     The name of the Teamcenter server's host machine.
    - LogFile         The full path of the Teamcenter server's log file.
    - Locale          The locale of this session.
    - TcServerID    The unique ID of this instance of the Teamcenter server.
    
    
    :var partialErrors: Partial errors or warnings resulting from the login attempt.
    """
    serverInfo: ServerInfo = None
    partialErrors: PartialErrors = None


@dataclass
class ClientCacheInfo(TcBaseObj):
    """
    Data for the requested cached features.
    
    :var features: The list of  features.
    :var partialErrors: Errors are return for features that do not exist or if there are other errors in getting data
    for a given service. The client ID in the partial error will be that of the feature name.
    """
    features: List[Feature] = ()
    partialErrors: PartialErrors = None


@dataclass
class Credentials(TcBaseObj):
    """
    The credentials needed to authenticate a user.
    
    :var user: The user's Teamcenter user name.
    :var password: The user's Teamcenter password or SSO token.
    :var group: The group ID for this session. If blank (""), the user's default group is assumed.
    :var role: The role the user is performing in the group. If blank (""), the user's default role in the group is
    assumed.
    :var locale: The locale to be used by the Teamcenter server process for this session. If blank (""), the server's
    default locale will be used.
    :var descrimator: Client defined identifier for this session. This argument is ignored when the client application
    is deployed in a 2Tier environment (IIOP communication).
    """
    user: str = ''
    password: str = ''
    group: str = ''
    role: str = ''
    locale: str = ''
    descrimator: str = ''


"""
Name/Value pairs of data related to the server. The following keys are valid:  Version, HostName, LogFile, Locale, TcServerID.
"""
ServerInfo = Dict[str, str]


"""
A map of FMS file tickes, the key is the name of the file, and the value is the FMS ticket.
"""
CacheTickets = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class FullTextResponse(TcBaseObj):
    """
    Represents the output data after retrieving the FullText dataset versions.
    
    :var revToFullText: A map( BusinessObject, list of BusinessObject) of Requirement Revision and its FullText dataset.
    :var objectPropValues: A map(BusinessObject, PropertyValues) of property name and values to be displayed.
    :var serviceData: The Service Data
    """
    revToFullText: RevisionToFullTextObjects = None
    objectPropValues: ObjectPropertyValues = None
    serviceData: ServiceData = None


@dataclass
class PropertyValues2(TcBaseObj):
    """
    A map of property name (key) and property values (values) in string format.
    
    :var propName: The property name.
    :var propValues: list of property values.
    """
    propName: str = ''
    propValues: List[str] = ()


@dataclass
class RequirementInput(TcBaseObj):
    """
    Represents the input data containing a list of input objects of type Requirement Revision or Arm0RequirementElement
    required to get the FullText dataset versions.
    
    :var operationType: The type of operation to perform. 
    If the value is 'DERIVED' then the properties of the FullText dataset for the derived Requirement specification
    will be retrieved. If no value is provided then all the properties of the FullText dataset for the Requirement
    specification along with body_text property will be retrieved.
    :var baseURL: This is address of the FMS servers that client machine uses.
    :var selectedObjects: The list of input objects of type Requirement Revision or Arm0RequirementElement to query its
    FullText dataset versions and its contents.
    """
    operationType: str = ''
    baseURL: str = ''
    selectedObjects: List[BusinessObject] = ()


"""
A list of property name and values to be displayed on the FullText dataset.
"""
ObjectPropertyValues = Dict[BusinessObject, List[PropertyValues2]]


"""
A map of Requirement Revision and its FullText dataset.
"""
RevisionToFullTextObjects = Dict[BusinessObject, List[BusinessObject]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import WorkspaceObject, Fnd0EditContext
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ReferencerEditContextsOutput(TcBaseObj):
    """
    This structure contains an input object and its referencing edit contexts.
    
    :var inputObject: The input WorkspaceObject.
    :var editContexts: The list of edit contexts referencing inputObject
    """
    inputObject: WorkspaceObject = None
    editContexts: List[Fnd0EditContext] = ()


@dataclass
class ReferencerEditContextsResponse(TcBaseObj):
    """
    This structure contains the input objects and the corresponding edit contexts that reference the input objects. Any
    partial errors that occur during the operation are returned in serviceData.
    
    :var serviceData: Contains any partial errors that occur during the operation.
    :var refnEditContextOutputs: A list of ReferencerEditContextsOutput.
    """
    serviceData: ServiceData = None
    refnEditContextOutputs: List[ReferencerEditContextsOutput] = ()



from __future__ import annotations

from typing import List
from tcsoa.base import TcService
from tcsoa.gen.Internal.Requirementsmanagement._2012_02.RequirementsManagement import TraceabilityMatrixInfo, TraceabilityMatrixResponse


class RequirementsManagementService(TcService):

    @classmethod
    def getTraceabilityMatrix(cls, inputs: List[TraceabilityMatrixInfo]) -> TraceabilityMatrixResponse:
        """
        This operation creates the traceability matrix between two BOM structures based on TraceLink  relation. A
        matrix will be generated that shows the source structure objects as rows in the matrix and columns as target
        structure. In the trace matrix if there is a link between row (object from the source structure) and the column
        (object from the target structure) then link count will be shown in the respective cell.  This operation can be
        performed between two different BOM structures or same structure. In this operation all children for selected
        source and target object will be loaded as rows and columns in the matrix and each cell defines the link
        between source row object and target column object. This matrix will show the results for TraceLink which are
        created from source object (object shown as rows) to target object( object shown as columns) and not include
        the link which created from object shown as column to object shown as rows.
        
        Use cases:
        You can create traceability matrix between source and target structure based on TraceLink relation. Source and
        target can be different structure or same structure.
        """
        return cls.execute_soa_method(
            method_name='getTraceabilityMatrix',
            library='Internal-Requirementsmanagement',
            service_date='2012_02',
            service_name='RequirementsManagement',
            params={'inputs': inputs},
            response_cls=TraceabilityMatrixResponse,
        )



from __future__ import annotations

from tcsoa.gen.AWS2._2017_06.UiConfig import GetUIConfigInput, GetUIConfigResponse
from typing import List
from tcsoa.base import TcService


class UiConfigService(TcService):

    @classmethod
    def getUIConfigs3(cls, getUiConfigsIn: List[GetUIConfigInput]) -> GetUIConfigResponse:
        """
        This operation returns information used by the client to render the User Interface. The information returned
        includes command and column configuration information. The returned information includes
        Fnd0CommandCollectionRel objects that associate the top level command collections to client scope in
        CommandConfigData structure.
        """
        return cls.execute_soa_method(
            method_name='getUIConfigs3',
            library='AWS2',
            service_date='2017_06',
            service_name='UiConfig',
            params={'getUiConfigsIn': getUiConfigsIn},
            response_cls=GetUIConfigResponse,
        )
